package com.miaozhen.config;

import com.miaozhen.util.OSinfoUtil;
import com.pythagorastech.tms.util.FileUtil;
import org.apache.ftpserver.FtpServer;
import org.apache.ftpserver.FtpServerFactory;
import org.apache.ftpserver.ftplet.Authority;
import org.apache.ftpserver.ftplet.FtpException;
import org.apache.ftpserver.ftplet.UserManager;
import org.apache.ftpserver.listener.ListenerFactory;
import org.apache.ftpserver.usermanager.PropertiesUserManagerFactory;
import org.apache.ftpserver.usermanager.impl.BaseUser;
import org.apache.ftpserver.usermanager.impl.WritePermission;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description :
 * Created by liyongbo on 2025-03-13 15:08.
 */
@Configuration
public class FTPServerConfig {

    private FtpServer ftpServer;

    @Bean
    @PostConstruct
    public FtpServer ftpServer() throws FtpException {
        // 创建 FTP 服务器工厂
        FtpServerFactory serverFactory = new FtpServerFactory();

        // 配置监听器（设置端口）
        ListenerFactory listenerFactory = new ListenerFactory();
        listenerFactory.setPort(2121); // 默认 FTP 端口是 21，这里使用 2121 避免冲突
        serverFactory.addListener("default", listenerFactory.createListener());

        // 配置用户管理器
        PropertiesUserManagerFactory userManagerFactory = new PropertiesUserManagerFactory();
        UserManager userManager = userManagerFactory.createUserManager();

        // 创建用户
        BaseUser user = new BaseUser();
        user.setName("admin"); // 用户名
        user.setPassword("admin"); // 密码
        String path = "";
        if(OSinfoUtil.isLinux()){
            path = "/tmp/ftp";
        }else{
            path = "D:\\tmp\\ftp";
        }
        checkFolder(path);
        user.setHomeDirectory(path); // 用户主目录

        // 设置用户权限
        List<Authority> authorities = new ArrayList<>();
        authorities.add(new WritePermission()); // 写权限
        user.setAuthorities(authorities);

        // 添加用户到用户管理器
        userManager.save(user);

        // 设置用户管理器到服务器工厂
        serverFactory.setUserManager(userManager);

        // 创建并启动 FTP 服务器
        ftpServer = serverFactory.createServer();
        ftpServer.start();
        return ftpServer;
    }

    @PreDestroy
    public void onDestroy() {
        if (ftpServer != null) {
            ftpServer.stop();
        }
    }

    public static void checkFolder(String sPath) {
        // 指定目录路径
        Path path = Paths.get(sPath);

        // 判断目录是否存在，不存在则创建
        if (!Files.exists(path)) {
            try {
                Files.createDirectories(path); // 创建目录（包括父目录）
                System.out.println("目录创建成功: " + path);
            } catch (IOException e) {
                System.err.println("目录创建失败: " + e.getMessage());
            }
        } else {
            System.out.println("目录已存在: " + path);
        }
    }
}
