package com.miaozhen.device;

import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.smi.PlaybackStatus;

public interface IDCServer {

    String getIP();

    public void setIP(String ip);

    /**
     * 获取服务器原始播放日志
     * 注意:
     * 1,Dolby需要提供ftp,调用成功后,Dolby服务器会将日志文件上传至ftp
     * 2,Doremi, Christie, Bacro需要提供日志存放一个临时目录
     * 3,其他服务器直接通过接口返回日志内容
     * @param ftpIp       FTP Ip(Dolby 专用)
     * @param user        FTP登录用户(Dolby 专用)
     * @param pass    FTP登录密码(Dolby 专用)
     * @param tempLogPath 日志临时存放目录(Doremi, Christie, Bacro专用)
     * @param startDate   开始日期
     * @param endDate     结束日期
     * @return 原始日志字符串
     * @throws Exception
     */
    String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
                       String startDate,String endDate) throws Exception;

    /**
     * Get PlayBackStatus . Will not succeed when server is busy.
     *
     * @return response
     * @throws Exception
     */
    PlaybackStatus getPlaybackStatus() throws Exception;

    /**
     * 服务器连接
     * @throws DeviceException
     */
    void connect() throws DeviceException;

    /**
     * Get ejectSPL with specified uuid. Will not succeed when server is busy.
     * Christie会用到
     * @param uuid
     * @return
     * @throws Exception
     */
    boolean ejectSPL(String uuid) throws Exception;

    String getServerType();

    void setConnected(boolean connected);

    boolean getConnected();

    /**
     * Reconnects to the device.
     *
     * @throws Exception
     */
    public void disconnect() throws DeviceException;
}
