package com.miaozhen.device;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.support.*;

/**
 * 服务器型号适配工厂类
 */
public class ServerFactory {

    public static ServerType[] getSupportTypes() {
        return ServerType.values();
    }

    public static IDCServer create(String type, String ip, String userName, String password) throws DeviceException {
        for (ServerType t : getSupportTypes()) {
            if (t.toString().equalsIgnoreCase(type)) {
                return t.newInstance(ip, userName, password);
            }
        }
        return null;
    }

    /**
     * 服务器型号
     */
    public enum ServerType {
        GDC("GDC", new String[] { "GDC" }, 49153, GDCServer.class),
        DOREMI("Doremi", new String[] { "Doremi" }, 11730, DoremiServer.class),
        DOLBY("Dolby", new String[] { "Dolby" }, 0, DolbyServer.class),
        Sony("Sony", new String[] { "Sony" }, 443, SonyServer.class),
        BARCO("Barco", new String[] { "Barco" }, 43758, BarcoServer.class),
        Christie("Christie", new String[] { "Christie" }, 0, ChristieServer.class),
        ChristieFusion("Christie Fusion IMB", new String[] { "Christie Fusion IMB" }, 0, ChristieFusionServer.class);

        private String name;

        private String[] model;

        private int port;

        private Class<?> clazz;

        private ServerType(String n, String[] m, int p, Class<?> c) {
            name = n;
            model = m;
            port = p;
            clazz = c;
        }

        public String[] getModel() {
            return model;
        }

        @Override
        public String toString() {
            return name;
        }

        public IDCServer newInstance(String ip, String userName, String password) throws DeviceException {
            try {
                Constructor<?> constructor = clazz.getDeclaredConstructor(new Class[] { String.class, int.class,String.class, String.class});
                return (IDCServer) constructor.newInstance(new Object[] { ip, port, userName, password });
            } catch (InstantiationException e) {
                // logger.debug("The class or its nullary constructor is not accessible.");
                throw new DeviceException(e);
            } catch (IllegalAccessException e) {
                // logger.debug("This Class represents an abstract class, an interface, an array class, a primitive type, or void; or if the class has no nullary constructor; or if the instantiation fails for some other reason.");
                throw new DeviceException(e);
            } catch (NoSuchMethodException e) {
                // logger.debug("This Class represents an abstract class, an interface, an array class, a primitive type, or void; or if the class has no nullary constructor; or if the instantiation fails for some other reason.");
                throw new DeviceException(e);
            } catch (SecurityException e) {
                // logger.debug("A matching method is not found.");
                throw new DeviceException(e);
            } catch (IllegalArgumentException e) {
                // logger.debug("Illegal argument for the constructor.");
                throw new DeviceException(e);
            } catch (InvocationTargetException e) {
                // logger.debug("The underlying constructor throws an exception.");
                throw new DeviceException(e);
            }
        }
    }
}
