package com.miaozhen.device.response;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import com.miaozhen.device.smi.PlaybackStatus;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class PlaybackStatusResponse extends Response {

	private PlaybackStatus playbackStatus;

	public PlaybackStatusResponse(String string) throws Exception {
		super(string);
		playbackStatus = new PlaybackStatus();
		handler = new DefaultHandler() {
			private String tag = null;

			@Override
			public void startElement(String uri, String localName,
					String qName, Attributes attributes) throws SAXException {
				if (qName.equals("playbackmode") || qName.equals("show_uuid")
						|| qName.equals("show_name")
						|| qName.equals("cpl_uuid") || qName.equals("cpl_name")) {
					tag = qName;
				} else if (qName.equals("status")) {
					String s = attributes.getValue("state");
					if (s.equals("STOPPED"))
						playbackStatus.setState(PlaybackStatus.STATE_STOPPED);
					else if (s.equals("PLAYING"))
						playbackStatus.setState(PlaybackStatus.STATE_PLAYING);
					else if (s.equals("PAUSED"))
						playbackStatus.setState(PlaybackStatus.STATE_PAUSED);
					else if (s.equals("READY"))
						playbackStatus.setState(PlaybackStatus.STATE_READY);
					else if (s.equals("CLEARED"))
						playbackStatus.setState(PlaybackStatus.STATE_WAITINGFORLOADING);
					else
						playbackStatus.setState(PlaybackStatus.STATE_ERROR);
				} else if (qName.equals("show_position")) {
					playbackStatus.setShowPosition(Integer.valueOf(attributes
							.getValue("played_duration")));
					playbackStatus.setShowDuration(Integer.valueOf(attributes
							.getValue("total_duration")));
				} else if (qName.equals("cpl_position")) {
					playbackStatus.setCPLIndex(Integer.valueOf(attributes
							.getValue("cpl_index")));
					playbackStatus.setCPLPostion(Integer.valueOf(attributes
							.getValue("played_duration")));
					playbackStatus.setCPLDuration(Integer.valueOf(attributes
							.getValue("total_duration")));
				}
			}

			@Override
			public void endElement(String uri, String localName, String qName)
					throws SAXException {
				if (qName.equals("playbackmode") || qName.equals("show_uuid")
						|| qName.equals("show_name")
						|| qName.equals("cpl_uuid") || qName.equals("cpl_name")) {
					tag = null;
				}
			}

			@Override
			public void characters(char[] ch, int start, int length)
					throws SAXException {
				if (tag != null) {
					if (tag.equals("playbackmode")) {
						String mode = new String(ch, start, length);
						if (mode.equals("2D")) {
							playbackStatus
									.setPlaybackMode(PlaybackStatus.PLAYBACKMODE_2D);
						} else if (mode.equals("3D")) {
							playbackStatus
									.setPlaybackMode(PlaybackStatus.PLAYBACKMODE_3D);
						} else if (mode.equals("DOLBY_3_D")) {
							playbackStatus
									.setPlaybackMode(PlaybackStatus.PLAYBACKMODE_DOLBY3D);
						} else {
							playbackStatus
									.setPlaybackMode(PlaybackStatus.PLAYBACKMODE_ERROR);
						}
					} else if (tag.equals("show_name")) {
						playbackStatus
								.setShowName(new String(ch, start, length));
					} else if (tag.equals("show_uuid")) {
						playbackStatus
								.setShowUUID(new String(ch, start, length));
					} else if (tag.equals("cpl_name")) {
						playbackStatus
								.setCPLName(new String(ch, start, length));
					} else if (tag.equals("cpl_uuid")) {
						playbackStatus
								.setCPLUUID(new String(ch, start, length));
					}
				}
			}
		};
		try {
			playbackStatus.setPlaybackMode(PlaybackStatus.PLAYBACKMODE_ERROR);
			playbackStatus.setState(PlaybackStatus.STATE_ERROR);
			parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(new InputSource(new StringReader(string)), handler);
		} catch (ParserConfigurationException e) {
			//logger.error(e.toString());
			throw new Exception("ParserConfigurationException", e);
		} catch (SAXException e) {
			//logger.error(e.toString());
			throw new Exception("SAXException", e);
		} catch (IOException e) {
			//logger.error(e.toString());
			throw new Exception("IOException", e);
		} finally {
			handler = null;
			parser = null;
		}
	}

	public PlaybackStatus getPlaybackStatus() {
		return playbackStatus;
	}

}