/**
 * 
 */
package com.miaozhen.device.response;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class Response {
	public static final int OK = 0;
	public static final int ERROR = 1;
	public static final int EXCEPTION = -1;


	protected String source;
	
	protected int status;
	protected String version;
	protected String error;
	protected SAXParser parser;
	protected DefaultHandler handler;

	/**
	 * Constructor of Response
	 * 
	 * @param string
	 *            The XML string to be converted.
	 */
	public Response(String string) {
		source = string;
		handler = new DefaultHandler() {
			private String tag = null;

			@Override
			public void startElement(String uri, String localName,
					String qName, Attributes attributes) throws SAXException {
				if (qName.equals("response")) {
					status = (attributes.getValue("status").equals("OK")) ? OK
							: ERROR;
					version = attributes.getValue("version");
				} else if (qName.equals("error")) {
					tag = qName;
				}
			}

			@Override
			public void endElement(String uri, String localName, String qName)
					throws SAXException {
				if (qName.equals("error")) {
					tag = null;
				}
			}

			@Override
			public void characters(char[] ch, int start, int length)
					throws SAXException {
				if (tag != null && tag.equals("error")) {
					error = new String(ch, start, length);
				}
			}
		};
		try {
			status = EXCEPTION;
			parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(new InputSource(new StringReader(source)), handler);
		} catch (SAXException e) {
		} catch (IOException e) {
		} catch (ParserConfigurationException e) {
		} finally {
			handler = null;
			parser = null;
		}
	}

	/**
	 * Get status of the response.
	 * 
	 * @return status
	 */
	public final int status() {
		return status;
	}

	/**
	 * Get version of the response.
	 * 
	 * @return version
	 */
	public final String version() {
		return version;
	}

	/**
	 * Get error message of the response.
	 * 
	 * @return error message
	 */
	public final String error() {
		return error;
	}
	
	public final String source() {
		return source;
	}
}
