package com.miaozhen.device.support;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.Socket;

import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.common.Constants;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.PlaybackStatusResponse;
import com.miaozhen.device.response.Response;
import com.miaozhen.device.response.StringResponse;
import com.miaozhen.device.smi.PlaybackStatus;
import org.crifst.project.tms.smi.barco.api.PlaybackControl;
import org.crifst.project.tms.smi.barco.api.SystemManagement;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@SuppressWarnings("restriction")
public class BarcoServer implements IDCServer {

    private String ip;
    private int port;
    private String userName;
    private String password;
    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }


    protected SystemManagement systemManagement;

    protected PlaybackControl playbackControl;

    private final Logger logger = LoggerFactory.getLogger(getClass());

    boolean isConnected = false;

    public BarcoServer(String ip, int port, String userName, String password) {
        this.ip = ip;
        this.port = port;
        this.userName = userName;
        this.password = password;
        isConnected = checkConnect();
    }


    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public boolean getConnected(){
        return isConnected;
    }

    @Override
    public void disconnect() throws DeviceException {

    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected)
            isConnected = checkConnect();
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.BARCO.toString();
    }

    private boolean checkConnect() {
        Socket socket = new Socket();

        try {
            socket.connect(new InetSocketAddress(ip, port));
        } catch (IOException e) {
            logger.info(" connect exception!!!");
            return false;
        } finally {
            try {
                socket.close();
                logger.info(" connect socket close!!!");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return true;
    }


    @Override
    public String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
                              String startDate,String endDate) throws Exception {
        if (isConnected) {
            systemManagement = new SystemManagement(ip, userName, password);
            String xml = systemManagement
                    .getauditlog(ftpIp, user, pass, startDate, Constants.startTime, endDate, Constants.endTime, "", "");
            StringResponse response = new StringResponse(xml, "auditlog_path");
            if (response.status() == Response.OK) {
                File xmlFile = new File(response.getResponse());
                SAXReader reader = new SAXReader();
                org.dom4j.Document document = reader.read(xmlFile);
                try {
                    xmlFile.deleteOnExit();
                } catch (Exception e) {
                    logger.error("delete file error", e);
                }
                return document.asXML();
            }
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
            playbackControl = new PlaybackControl(ip, userName, password);
            String xml = playbackControl.getplaybackstatus();
            // 如果发生以下问题 先重试3次 避免TMS屏幕界面频繁的出现连接丢失情况
            // <response status="ERROR" version="2">
            //   <error>icmp - insufficient user rights - command rejected</error>
            // </response>
            if ((xml.toLowerCase()).contains("insufficient")) {
//                logger.info(
//                        "BarcoServer getPlaybackStatus occur error, xml: {}, ip: {}, userName: {}, password: {}",
//                        xml, ip, userName, password);
                for (int i = 0; i < 3; i++) {
//                    logger.info("BarcoServer getPlaybackStatus occur error, retry: {}/3", i + 1);
                    playbackControl = new PlaybackControl(ip, userName, password);
                    xml = playbackControl.getplaybackstatus();
                    if (!(xml.toLowerCase()).contains("insufficient"))
                        break;
                }
            }

//            logger.info("BarcoServer getPlaybackStatus, result : {}", xml);
            PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
            if (response.status() == Response.OK)
                return response.getPlaybackStatus();
        }
        return null;
    }
}
