package com.miaozhen.device.support;

import java.io.*;
import java.net.Socket;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.common.Constants;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.PlaybackStatusResponse;
import com.miaozhen.device.response.Response;
import com.miaozhen.device.response.StringResponse;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.christie.PlaybackControlChris;
import com.miaozhen.device.support.christie.SystemManagementChris;
import com.miaozhen.device.support.christie.basecmd.UtilTMSChris;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;

public class ChristieServer implements IDCServer {

    private String ip;

    protected SystemManagementChris systemManagement;
    protected PlaybackControlChris playbackControl;

    private Socket socket = null;

    private UtilTMSChris util = new UtilTMSChris();

    private final Logger logger = LoggerFactory.getLogger(getClass());

    private boolean isConnected = false;

    public ChristieServer(String ip, int port, String userName, String password) {
        this.ip = ip;
        isConnected = checkConnect();
        if (isConnected) {
            this.socket = new Socket();
            util.login(this.socket, ip, 4502);
        }
        systemManagement = new SystemManagementChris();
        playbackControl = new PlaybackControlChris();

    }


    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (process != null)
                process.destroy();
        }
        return false;
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public boolean getConnected(){
        return isConnected;
    }

    @Override
    public void disconnect() throws DeviceException {

    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected) {
            isConnected = checkConnect();
        }
        if (isConnected) {
            if (socket == null || socket.isClosed()) {
                socket = new Socket();
            }
            if (util.login(this.socket, ip, 4502)) {
                systemManagement.setSocket(socket);
                playbackControl = new PlaybackControlChris();
            } else {
                socket = new Socket();
                isConnected = false;
            }
        }
    }


    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
                              String startDate,String endDate) throws Exception {
        if (isConnected) {
            if(socket!=null&&systemManagement.getSocket()==null){
                systemManagement.setSocket(socket);
            }
            String xml = systemManagement
                    .getauditlogs(ip, tempLogPath, startDate, Constants.startTime, endDate, Constants.endTime, "", "");
            StringResponse response = new StringResponse(xml, "auditlog_path");
            if (response.status() == Response.OK) {
                File xmlFile = new File(response.getResponse());
                StringBuffer sb = new StringBuffer();
                FileReader reader = new FileReader(xmlFile);
                BufferedReader br = new BufferedReader(reader);
                String str = null;
                while ((str = br.readLine()) != null) {
                    sb.append(str);
                }
                try {
                    xmlFile.deleteOnExit();
                } catch (Exception e) {
                    logger.error("delete file error", e);
                }
                return sb.toString();
            }
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
            if(playbackControl==null){
                new PlaybackControlChris();
            }
            if(socket!=null&&playbackControl.getSocket()==null){
                playbackControl.setSocket(socket);
            }
            String xml = playbackControl.getplaybackstatus(ip);
//            logger.info("ChristieServer getPlaybackStatus, result : {}", xml);
            PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
            if(response.getPlaybackStatus().getState() == -1){
                for (int i = 0; i < 3; i++) {
                    logger.info("getPlaybackStatus retry {}/3",i+1);
                    xml = playbackControl.getplaybackstatus(ip);
                    response = new PlaybackStatusResponse(xml);
                    if (response.status() == Response.OK && response.getPlaybackStatus().getState() != -1)
                        break;
                }
            }

            if (response.status() == Response.OK) {
                return response.getPlaybackStatus();
            }
        }
        return null;
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        if (isConnected) {
            if(playbackControl==null){
                new PlaybackControlChris();
            }
            String result = playbackControl.ejectSPL();
            logger.info("ChristieServer ejectSPL, result : {}", result);
            Response response = new Response(result);
            if (response.status() == Response.OK)
                return true;
        }
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.Christie.toString();
    }
}
