package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.common.Constants;
import com.miaozhen.device.expection.CommunicationException;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.PlaybackStatusResponse;
import com.miaozhen.device.response.Response;
import com.miaozhen.device.response.StringResponse;
import com.miaozhen.device.smi.PlaybackStatus;
import org.crifst.smi.doremi.api.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;


public class DoremiServer implements IDCServer {

    private final Logger logger = LoggerFactory.getLogger(getClass());

    public final static int SOCKET_TIMEOUT = 3000;

    protected final byte[] frame =
            {0x06, 0x0E, 0x2B, 0x34, 0x02, 0x05, 0x01, 0x0A, 0x0E, 0x10, 0x01, 0x01, 0x01, 0x00, 0x00,
                    0x00, (byte) 0x83, 0x00, 0x00, 0x00, 0x74, 0x24, 0x75, (byte) 0x87};

    protected transient Socket socket;

    protected String ip;
    private int port;

    protected boolean flag = false;

    private boolean isConnected = false;

    protected SystemManagement systemManagement;

    protected PlaybackControl playBackControl;


    public DoremiServer(String ip, int port, String userName, String password) {
        this.ip = ip;
        this.port = port;
        isConnected = checkConnect();
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public boolean getConnected(){
        return isConnected;
    }

    @Override
    public void disconnect() throws DeviceException {
        release(systemManagement);
    }

    private void release(Object o) {
        if (o != null)
            o = null;
    }
    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (process != null)
                process.destroy();
        }
        return false;
    }

    protected void _connect() throws CommunicationException {
        try {
            socket = new Socket(ip, port);
            socket.setKeepAlive(true);
            socket.setSoTimeout(SOCKET_TIMEOUT);
        } catch (UnknownHostException e) {
            throw new CommunicationException(e);
        } catch (IOException e) {
            throw new CommunicationException(e);
        }
    }

    protected void _disconnect() {
        try {
            if (socket != null) {
                socket.close();
                socket = null;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected)
            isConnected = checkConnect();
        if (isConnected) {
            systemManagement = new SystemManagement();
            playBackControl = new PlaybackControl();
        }
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.DOREMI.toString();
    }

    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
                              String startDate,String endDate) throws Exception {
        if (isConnected) {
            systemManagement = new SystemManagement();
            String xml = systemManagement
                    .getauditlogs(ip, tempLogPath, startDate, Constants.startTime, endDate, Constants.endTime, null, null);
            StringResponse response = new StringResponse(xml, "auditlog_path");
            if (response.status() == Response.OK) {
                File xmlFile = new File(response.getResponse());
                StringBuffer sb = new StringBuffer();
                FileReader reader = new FileReader(xmlFile);
                BufferedReader br = new BufferedReader(reader);
                String str = null;
                while ((str = br.readLine()) != null) {
                    sb.append(str);
                }
                //清除临时文件
                try {
                    xmlFile.deleteOnExit();
                } catch (Exception e) {
                    logger.error("delete file error", e);
                }
                //某些版本的doremi会有注释信息,这里需要特殊处理下
                String tempResult = sb.toString();
                String fin_tempResult="";
                try{
                    fin_tempResult=tempResult
                            .substring(tempResult.indexOf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"));
                    return fin_tempResult;
                }catch(Exception e){
                    fin_tempResult=tempResult
                            .substring(tempResult.indexOf("<?xml version=\"1.0\" encoding=\"utf-8\"?>"));
                    return fin_tempResult;
                }
            }
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
            playBackControl = new PlaybackControl();
            String xml = playBackControl.getplaybackstatus(ip);
//            logger.info("DoremiServer getPlaybackStatus, result : {}", xml);
            PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
            if (response.status() == Response.OK)
                return response.getPlaybackStatus();
        }
        return null;
    }
}
