package com.miaozhen.device.support;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.common.Constants;
import com.miaozhen.device.expection.CommunicationException;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.response.PlaybackStatusResponse;
import com.miaozhen.device.response.Response;
import com.miaozhen.device.smi.PlaybackStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.UnknownHostException;
public class GDCServer implements IDCServer {

    protected transient Socket socket;

    protected String ip;

    protected int port;
    boolean isConnected = false;

    private final Logger logger = LoggerFactory.getLogger(getClass());

    protected static final byte[] GDC_HEADER =
            {0x06, 0x0E, 0x2B, 0x34, 0x02, 0x05, 0x01, 0x01, 0x0F, 0x15, 0x01, 0x10, 0x00, 0x00, 0x00,
                    0x00};

    public GDCServer(String ip, int port, String userName, String password) {
        this.ip = ip;
        this.port = port;
        isConnected = checkConnect();
    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + ip + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + ip + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    return true;
                } else {
                    return false;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    protected void _connect() throws DeviceException {
        try {
            socket = new Socket();
            SocketAddress socketAddress = new InetSocketAddress(ip, port); // 获取sockaddress对象
            socket.connect(socketAddress, 3000);
            socket.setKeepAlive(true);
            socket.setSoTimeout(60000);
        } catch (UnknownHostException e) {
            throw new CommunicationException(e);
        } catch (IOException e) {
            throw new CommunicationException(e);
        }
    }

    protected void _disconnect() throws DeviceException {
        try {
            if (socket != null) {
                socket.close();
                socket = null;
            }
        } catch (IOException e) {
            throw new CommunicationException(e);
        }
    }

    @Override
    public void setConnected(boolean connected) {
        isConnected = connected;
    }

    @Override
    public boolean getConnected(){
        return isConnected;
    }

    @Override
    public void disconnect() throws DeviceException {

    }

    protected String transfer(String data) throws IOException, Exception {
        OutputStream os = null;
        InputStream is = null;
        ByteArrayOutputStream baos = null;
        try {
            _connect();
            os = socket.getOutputStream();
            is = socket.getInputStream();
            os.write(GDC_HEADER);
            os.write(calculate(data));
            os.write(data.getBytes());
            socket.shutdownOutput();
            baos = new ByteArrayOutputStream();
            for (int c; (c = is.read()) != -1; baos.write(c))
                ;
            byte[] b = baos.toByteArray();
            is.close();
            os.close();
            if (b.length <= 0 || b.length < 20) {
                b = null;
                return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>send and recieve failed!</error>\n</response>\n";
            }
            String response = new String(b, 20, b.length - 20);
            if (response.equals("false")) {
                b = null;
                return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>send and recieve failed!</error>\n</response>\n";
            }
            b = null;
            return response;
        } finally {
            if (baos != null) {
                baos.close();
                baos = null;
            }
            if (os != null) {
                os.close();
                os = null;
            }
            if (is != null) {
                is.close();
                is = null;
            }
            _disconnect();
        }
    }

    protected byte[] calculate(String data) {
        byte[] length = new byte[4];
        int value = data.length();
        length[0] = (byte) 0x83;
        for (int i = 3; i > 0; --i) {
            length[i] = (byte) (value & 0xFF);
            value >>= 8;
        }
        return length;
    }

    @Override
    public void connect() throws DeviceException {
        if (!isConnected) {
            isConnected = checkConnect();
        }
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.GDC.toString();
    }

    @Override
    public String getIP() {
        return ip;
    }

    @Override
    public void setIP(String ip) {
        this.ip = ip;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
                              String startDate,String endDate) throws Exception {
        if (isConnected) {
            String cmd =
                    "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<command version=\"2\" cmd=\"GET_CONTENT_LOGS\" >\n\t<log date=\""
                            + startDate + "\" time=\"" + Constants.startTime + "\" end_date=\"" + endDate
                            + "\" end_time=\"" + Constants.endTime + "\" />\n</command>";
            return transfer(cmd);
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if (isConnected) {
            String cmd =
                    "<?xml version=\"1\" encoding=\"UTF-8\" ?>\n<command version=\"2\" cmd=\"GET_PLAYBACK_STATUS\"/>";
            String xml = transfer(cmd);
//            logger.info("GDCServer getPlaybackStatus, result : {}", xml);
            if (xml != null) {
                PlaybackStatusResponse response = new PlaybackStatusResponse(xml);
                if (response.status() == Response.OK)
                    return response.getPlaybackStatus();
            }
        }
        return null;
    }
}
