package com.miaozhen.device.support;

import java.io.*;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.ServerFactory;
import com.miaozhen.device.common.Constants;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.device.expection.SonyErrorResponseException;
import com.miaozhen.device.response.SonyStringResponse;
import com.miaozhen.device.response.StringResponse;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.smi.JAXBUtils;
import com.miaozhen.device.support.smi.playbackstatus.sonyserver.ShowStatus;
import com.miaozhen.device.support.smi.playbackstatus.sonyserver.ShowStatusDetail;
import com.miaozhen.device.util.MySecureProtocolSocketFactory;
import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.barco.dual.projector.EPlatform;
import com.barco.dual.projector.OSinfo;

public class SonyServer implements IDCServer {

    private final Logger logger = LoggerFactory.getLogger(getClass());

    protected String ip;

    protected static int port;

    protected HttpClient httpClient = null;

    public static MultiThreadedHttpConnectionManager connectionManager = null;

    private boolean isLoginSuccess = false;

    private String userName = null;

    private String password = null;

    private String sonyIP;

    private ShowStatus showStatus;

    public SonyServer(String ip, int port, String userName, String password)
        throws DeviceException {
        this.sonyIP = ip;
        this.userName = userName;
        this.password = password;
        this.ip = "https://" + ip;
        this.port = port;
//        logger.info("checkConnect() is:{}",checkConnect());
        if (checkConnect())
            Sony_connect(getInstance());
    }

    public boolean checkConnect() {
        EPlatform OSInfo = OSinfo.getOSname();
        Runtime runtime = Runtime.getRuntime();
        Process process = null;
        try {
            if (OSInfo.equals(EPlatform.Windows)) {
                process = runtime.exec("ping " + this.sonyIP + " -n 1 -w 3");

            } else {
                process = runtime.exec("ping " + this.sonyIP + " -c 1 -w 3");

            }
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line = null;
            StringBuffer sb = new StringBuffer();
            while ((line = br.readLine()) != null) {
                sb.append(line);
            }
            is.close();
            isr.close();
            br.close();
            if (null != sb && !sb.toString().equals("")) {
                if (sb.toString().toLowerCase().indexOf("ttl") > 0) {
                    logger.info("sony 连接成功{}", this.sonyIP);
                    return true;
                } else {
                    logger.info("sony 连接失败{}", this.sonyIP);
                    return false;
                }
            }
        } catch (Exception e) {
            logger.info("SonyServer checkConnect exception : {}", e);
            return false;
        } finally {
            if (process != null) {
                process.destroy();
            }
        }
        return false;
    }

    @Override
    public void setConnected(boolean connected) {
        isLoginSuccess = connected;
    }

    @Override
    public boolean getConnected(){
        return isLoginSuccess;
    }

    public HttpClient getInstance() {
        if (connectionManager == null) {
            int maxThreadsTotal = 200;
            int maxThreadsPerHost = 50;
            connectionManager = new MultiThreadedHttpConnectionManager();
            HttpConnectionManagerParams params = connectionManager.getParams();
            params.setMaxTotalConnections(maxThreadsTotal);
            if (maxThreadsTotal > maxThreadsPerHost) {
                params.setDefaultMaxConnectionsPerHost(maxThreadsPerHost);
            } else {
                params.setDefaultMaxConnectionsPerHost(maxThreadsTotal);
            }
            connectionManager.setParams(params);
            connectionManager.closeIdleConnections(0);
            connectionManager.getParams().setSoTimeout(10000);
            connectionManager.getParams().setConnectionTimeout(5000);
        }
        if (httpClient == null) {
            // connectionManager.setMaxTotalConnections (5);//总的连接数
            // connectionManager.setMaxConnectionsPerHost (10);//每个host的最大连接数
            httpClient = new HttpClient(connectionManager);
            ProtocolSocketFactory fcty = new MySecureProtocolSocketFactory();
            Protocol.registerProtocol("https", new Protocol("https", fcty, port));
            // httpClient.getHttpConnectionManager().getParams()
            // .setConnectionTimeout(3000);
            httpClient.getHttpConnectionManager().getParams().setSoTimeout(3000);
            httpClient.getParams().setConnectionManagerTimeout(10000);
            httpClient.getHttpConnectionManager().getParams()
                .setConnectionTimeout(5000);// 由于要上传的文件可能比较大
            // ,
            // 因此在此设置最大的连接超时时间
        }
        return httpClient;
    }

    public void Sony_connect(HttpClient httpClient) throws DeviceException {
        if (userName == null || userName.equals("")) {
            userName = "super";
        }
        if (password == null || password.equals("")) {
            password = "0955";
        }
        GetMethod method = new GetMethod(ip + "/login");

        try {
            method.setDoAuthentication(true);
            method.addRequestHeader("Authorization", "Basic c3VwZXI6MDk1NQ==");
            method.setRequestHeader("User", userName);
            method.setRequestHeader("Password", password);
            method.setRequestHeader("Connection", "close");
            httpClient.executeMethod(method);
            String userInfo = method.getResponseBodyAsString();
            if (userInfo.contains("UserInfo"))
                isLoginSuccess = true;
            else if (userInfo.contains("ErrorResponse")) {
                isLoginSuccess = false;
                StringResponse sr = new StringResponse(userInfo, "Code");
                String errorCode = sr.getResponse();
                if (errorCode.equals("202"))
                    throw new SonyErrorResponseException(
                        "Login :Not Authorized. The user does not have the appropriate credentials to make the request.   ");
                else if (errorCode.equals("204"))
                    throw new SonyErrorResponseException(
                        "Login :Access failed, Failed to access a file, folder, or any object.");
                else if (errorCode.equals("216"))
                    throw new SonyErrorResponseException("Login :Invalid message header");
            }
        } catch (Exception e) {
            logger.error("SonyServer connect exception : {}", e);
        } finally {
            method.releaseConnection();
        }

    }

    public String getResponseBody(HttpMethod method) throws HttpException, IOException {
//        logger.info("getResponseBody isLoginSuccess is:{}",isLoginSuccess);
        String responseBody = null;
        try {
            if (!isLoginSuccess) {
                if (checkConnect())
                    Sony_connect(getInstance());
                else
                    return null;
            }
            //设置120秒超时
            method.getParams().setParameter(HttpMethodParams.SO_TIMEOUT, 1000 * 120);
            int code = httpClient.executeMethod(method);
            if (code == 200) {
                responseBody = method.getResponseBodyAsString();
                if (responseBody.contains("<ErrorResponse>")) {
                    {
                        SonyStringResponse sr = new SonyStringResponse(responseBody, "Code");
                        String codeStr = sr.getResponse();
                        String errorCode = codeStr.replace("<Code>", "").replace("</Code>", "");
                        if (errorCode.equals("100")) {
                            return responseBody;
                        } else {
//                            logger.info(
//                                "SonyServer getResponseBody serverIP : {}, method : {}, errorCode : {},  responseBody : {} ",
//                                sonyIP, method.getURI(), responseBody);
                            return null;
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error("SonyServer getResponseBody serverIP : {}, error", sonyIP, e);
        } finally {
            method.releaseConnection();
        }
        return responseBody;
    }

    @Override
    public void connect() throws DeviceException {
    }

    @Override
    public boolean ejectSPL(String uuid) throws Exception {
        return false;
    }

    @Override
    public String getServerType() {
        return ServerFactory.ServerType.Sony.toString();
    }

    @Override
    public void disconnect() throws DeviceException {
        if (checkConnect()) {
            GetMethod method = new GetMethod(ip + "/logout");
            int code;
            try {
                method.setRequestHeader("Connection", "close");
                code = getInstance().executeMethod(method);
                if (code == 202 || code == 204 || code == 216)
                    isLoginSuccess = false;
                else
                    isLoginSuccess = true;
            } catch (Exception e) {
                logger.error("SonyServer disconnect exception : {}", e);
            }

            method.releaseConnection();
        }
    }

    @Override
    public String getIP() {
        return sonyIP;
    }

    @Override
    public void setIP(String ip) {
        this.sonyIP = sonyIP;
    }

    @Override
    public String getAuditLog(String ftpIp, String user, String pass, String tempLogPath,
        String startDate,String endDate) throws Exception {
//        logger.info("getAuditLog isLoginSuccess is:{}",isLoginSuccess);
        if(isLoginSuccess){
            String start = startDate.replaceAll("-", "") + Constants.startTime.replaceAll(":", "");
            String end = endDate.replaceAll("-", "") + Constants.endTime.replaceAll(":", "");
            String url = String
                    .format("%s/command/sms/auditlog/report/directget?start=%s&end=%s", ip, start, end);
            GetMethod getMethod = new GetMethod(url);
            getMethod.setRequestHeader("Connection", "close");
            getMethod.getParams().setParameter(HttpMethodParams.SO_TIMEOUT, 1000 * 60);
            String xml = getResponseBody(getMethod);
            if (xml == null)
                return null;
            return xml;
        }
        return null;
    }

    @Override
    public PlaybackStatus getPlaybackStatus() throws Exception {
        if(isLoginSuccess) {
            GetMethod method = new GetMethod(ip + "/playback/showstatus");
            method.setRequestHeader("Connection", "close");
            String responseBody = getResponseBody(method);
//        logger.info("SonyServer getPlaybackStatus serverIP : {}, result : {}", sonyIP, responseBody);
            if (responseBody == null || !responseBody.contains("ShowStatus"))
                return null;
            SonyStringResponse stringResponse = new SonyStringResponse(responseBody, "ShowStatus");
            ShowStatus showStatus = JAXBUtils.unmarshal(stringResponse.getResponse(), ShowStatus.class);
            this.showStatus = showStatus;
            PlaybackStatus playbackStatus = new PlaybackStatus();
            switch (showStatus.getPlayState()) {
                case "PLAYING":
                    playbackStatus.setState(PlaybackStatus.STATE_PLAYING);
                    break;
                case "STOPPED":
                    playbackStatus.setState(PlaybackStatus.STATE_STOPPED);
                    break;
                case "COMPLETED":
                    playbackStatus.setState(PlaybackStatus.STATE_STOPPED);
                    break;
                case "INTERUPTED":
                    playbackStatus.setState(PlaybackStatus.STATE_STOPPED);
                    break;
                case "NONE":
                    playbackStatus.setState(PlaybackStatus.STATE_READY);
                    break;
                case "PAUSED":
                    playbackStatus.setState(PlaybackStatus.STATE_PAUSED);
                    break;
                case "ERROR":
                    playbackStatus.setState(PlaybackStatus.STATE_ERROR);
                    break;
                default:
                    playbackStatus.setState(PlaybackStatus.STATE_ERROR);
                    break;
            }
            playbackStatus.setPlaybackMode(showStatus.getPlayBackMode());
            ShowStatusDetail showStatusDetail = showStatus.getShowStatusDetail();
            String playType = showStatusDetail.getType();
            playbackStatus.setPlayType(playType);
            playbackStatus.setShowUUID(showStatusDetail.getId());
            playbackStatus.setShowPosition((int) showStatusDetail.getElapsedTime() / 1000);
            playbackStatus.setShowDuration((int) showStatusDetail.getTotalDuration() / 1000);
            playbackStatus.setShowName("");
            if (playType.equals("SPL")) {
                playbackStatus.setCPLUUID(showStatusDetail.getCurrentEventId());
                playbackStatus.setCPLDuration((int) showStatusDetail.getRemainingTime() / 1000
                        + (int) showStatusDetail.getElapsedTime() / 1000);
                playbackStatus.setCPLPostion((int) showStatusDetail.getElapsedTime() / 1000);
            } else {
                playbackStatus.setCPLUUID(showStatusDetail.getId());
                playbackStatus.setCPLDuration((int) showStatusDetail.getTotalDuration() / 1000);
                playbackStatus.setCPLPostion((int) showStatusDetail.getElapsedTime() / 1000);
            }
            return playbackStatus;
        }
        return null;
    }
}
