package com.miaozhen.device.support.christie.basecmd;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.crifst.smi.christie.basecmd.CRC16;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public abstract class BaseChristieSeries2SocketCmdTMS {

	private int timeOut;
	private int soTimeOut;
	protected Socket socket = null;
	protected Socket socket1 = null;
	private InputStream is;
	private OutputStream os;
	private String recvXml;
	private final byte MessageStart[] = { 123, 2, 52, 43 };
	private byte HeaderData[] = { 1, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	private final Logger logger = LoggerFactory.getLogger(getClass());

	public BaseChristieSeries2SocketCmdTMS(Socket socket, String host,
			int tcpPort, byte ObjectID) throws IOException {
		this.timeOut = 10000;
		this.soTimeOut = 5000;
		this.socket = socket;
		this.is = this.socket.getInputStream();
		this.os = this.socket.getOutputStream();
		this.HeaderData[3] = ObjectID;
	}

	public BaseChristieSeries2SocketCmdTMS(Socket socket, byte ObjectID)
			throws IOException {
		this.socket = socket;
		this.is = this.socket.getInputStream();
		this.os = this.socket.getOutputStream();
		this.HeaderData[3] = ObjectID;
	}

	protected abstract byte[] createCmdData();

	protected abstract String getResult(byte abyte0[]);

	protected abstract String getErrorMsg(int i);

	public void send(byte bytes[]) throws Exception {
		this.os.write(bytes);
		this.os.flush();
	}

	public byte[] receive(int length) throws Exception {
		byte[] result = new byte[length];
		int bufsize = (length < this.socket.getReceiveBufferSize()) ? length
				: this.socket.getReceiveBufferSize();
		int size = 0;
		while (size < length) {
			int s = length - size;
			byte[] buffer = new byte[(bufsize > s) ? s : bufsize];
			int readsize = this.is.read(buffer);
			System.arraycopy(buffer, 0, result, size, readsize);
			size += readsize;
		}
		return result;
	}

	public void execute() {
		try {
			byte[] cmdData = createCmdData();
			byte[] sendLength = UtilTMSChris.encodeDefiniteFormBER(cmdData.length);
			byte[][] cc = { sendLength, cmdData };
			 new CRC16().getCrc(cmdData);
			 new CRC16().getCrc(UtilTMSChris.byteArrayCopy(cc));

			byte[][] cmds = { this.MessageStart, this.HeaderData, sendLength,
					cmdData };
			byte[] cmdSend = UtilTMSChris.byteArrayCopy(cmds);

			send(cmdSend);
			byte[] receiveHeadData;
			int errorCode = 0;
			int length;
			byte[] elements;
			do {
//				if ((receive(1)[0] == this.MessageStart[0])
//						&& (receive(1)[0] == this.MessageStart[1])
//						&& (receive(1)[0] == this.MessageStart[2])
//						&& (receive(1)[0] == this.MessageStart[3]))
//					;
				receive(4);
				receiveHeadData = receive(14);

				byte[] error = { receiveHeadData[4], receiveHeadData[5] };
				errorCode = UtilTMSChris.littleEndianToInt(error);

				byte[] ber1 = receive(1);
				int berLength = UtilTMSChris.getDefiniteFormBERLength(ber1[0]);
				byte[] ber2 = receive(berLength - 1);
				byte[][] b = { ber1, ber2 };
				byte[] ber = UtilTMSChris.byteArrayCopy(b);

				length = UtilTMSChris.decodeDefiniteFormBER(ber);

				elements = receive(length);
			} while ((receiveHeadData[3] != this.HeaderData[3])
					|| ((receiveHeadData[3] == 21) && (length > 2500)));
			if (errorCode == 0) {
				this.recvXml = getResult(elements);
				return;
			}
			String errorMsg;
			if (errorCode >= 122)
				errorMsg = getGobalErrorMsg(errorCode);
			else {
				errorMsg = getErrorMsg(errorCode);
			}
			this.recvXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ errorMsg + "</error>\n</response>";
		} catch (Exception e) {
			e.printStackTrace();
			try {
				logger.error("execute exception {}",e.getMessage());
				this.socket.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
	}

	private String getGobalErrorMsg(int errorCode) {
		String result = "";
		switch (errorCode) {
		case 125:
			result = "ERROR_API_NOTFOUND";
			break;
		case 124:
			result = "ERROR_API_INVALID_SECURIY_LEVEL";
			break;
		case 123:
			result = "ERROR_API_NO_SECURIY_LEVEL";
			break;
		case 122:
			result = "ERROR_API_CURRENTLY_UPGRADING";
		}
		return result;
	}

	public String getResponse() {
		return this.recvXml;
	}
}
