package com.miaozhen.device.support.christie.playbackcontrol;

import java.io.IOException;
import java.net.Socket;
import java.util.List;

import com.christie.dcinema.xml.statusitem.Status;

public class GetPlaybackStatusTMS extends GetStatusTMS {

    public static enum SmiPlaybackStateTypeEnum {
        PLAYING, PAUSED, STOPPED, ERROR;

        public String value() {
            return name();
        }

        public static SmiPlaybackStateTypeEnum fromValue(String v) {
            return valueOf(v);
        }
    }

    private static Integer[] keys = { Integer.valueOf(300), Integer.valueOf(302), Integer.valueOf(303), Integer.valueOf(301), Integer.valueOf(308),
            Integer.valueOf(313), Integer.valueOf(314) };

    private String showUUID;

    private long playPosition;

    private boolean pause = false;

    public GetPlaybackStatusTMS(Socket socket, String host, int tcpPort) throws IOException {
        super(socket, host, tcpPort, keys);
    }

    public GetPlaybackStatusTMS(Socket socket) throws IOException {
        super(socket, keys);
    }

    protected String getResult(List<Status> statuslist) {
        String xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >";

        String playbackMode = "";
        String state = SmiPlaybackStateTypeEnum.ERROR.name();
        String showId = "";
        String showName = "";
        String showPlayedDuration = "0";
        String showTotalDuration = "0";
        String cplId = "";
        String cplName = "";
        String cplPlayedDuration = "0";
        String cplTotalDuration = "0";
        for (Status status : statuslist)
            switch (status.getId()) {
                case 308:
                    playbackMode = (Boolean.valueOf(status.getValue()).booleanValue()) ? "LOOP" : "NO LOOP";
                    break;
                case 300:
                    switch (Integer.valueOf(status.getValue()).intValue()) {
                        case 0:
                            state = SmiPlaybackStateTypeEnum.STOPPED.name();
                            break;
                        case 1:
                            state = SmiPlaybackStateTypeEnum.PLAYING.name();
                            break;
                        case 2:
                            state = SmiPlaybackStateTypeEnum.PAUSED.name();
                            this.pause = true;
                            break;
                    }
                    break;
                case 301:
                    showId = status.getValue().replaceAll("playlist_", "urn:uuid:");
                    this.showUUID = showId;
                    break;
                case 302:
                    showPlayedDuration = status.getValue();
                    this.playPosition = Long.valueOf(showPlayedDuration).longValue();
                    break;
                case 303:
                    showTotalDuration = status.getValue();
                    break;
                case 313:
                    cplId = status.getValue();
                    break;
                case 314:
                    cplPlayedDuration = ((status.getValue() == null) || (status.getValue().isEmpty())) ? "0" : status.getValue();
                    break;
            }
        // try {
        // GetPlaybackShowInfo cmd = new GetPlaybackShowInfo(this.socket,
        // showId, cplId);
        // cmd.execute();
        // GetPlaybackShowInfo.ShowInfo showInfo = cmd.getShowInfo();
        // showName = (showInfo.getShowName() == null) ? "" : showInfo
        // .getShowName();
        // cplName = (showInfo.getCplName() == null) ? "" : showInfo
        // .getCplName();
        // cplTotalDuration = (showInfo.getCplDuration() == null) ? "0"
        // : showInfo.getCplDuration();
        // } catch (IOException e) {
        // e.printStackTrace();
        // }
        xml = xml + "\n\t<playbackmode>" + playbackMode + "</playbackmode>";
        xml = xml + "\n\t<status state=\"" + state + "\" >";
        xml = xml + "\n\t\t<show_uuid>" + showId + "</show_uuid>";
        xml = xml + "\n\t\t<show_name>" + showName + "</show_name>";
        xml = xml + "\n\t\t<show_position total_duration=\"" + (Long.valueOf(showTotalDuration).longValue() / 1000L) + "\" played_duration=\""
                + (this.playPosition / 1000L) + "\" />";
        xml = xml + "\n\t\t<cpl_uuid>" + cplId + "</cpl_uuid>";
        xml = xml + "\n\t\t<cpl_name>" + cplName + "</cpl_name>";
        xml = xml + "\n\t\t<cpl_position cpl_index=\"0\" total_duration=\"" + (Long.valueOf(cplTotalDuration).longValue()) + "\" played_duration=\""
                + (Long.valueOf(cplPlayedDuration).longValue() / 1000L) + "\" />";
        xml = xml + "\n\t</status>\n</response>";
        return xml;
    }

    public String getShowUUID() {
        return this.showUUID;
    }

    public long getPlayPosition() {
        return this.playPosition;
    }

    public boolean isPause() {
        return this.pause;
    }
}
