package com.miaozhen.device.support.christie.systemmanagement;

import java.io.*;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import com.miaozhen.device.support.christie.basecmd.BaseChristieSeries2SocketCmdTMS;
import org.crifst.smi.christie.basecmd.IntegerElement;
import org.crifst.smi.christie.basecmd.MessageData;
import org.crifst.smi.christie.basecmd.StringElement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



public class GetLogsTMS extends BaseChristieSeries2SocketCmdTMS {

    private String start_date;
    private String end_date;
    private String path;

    private Logger logger = LoggerFactory.getLogger(getClass());

    public GetLogsTMS(Socket socket, String host, int tcpPort, String path, String start_date,
        String start_time, String end_date, String end_time, String CPL_UUID, String KDM_UUID)
        throws IOException {
        super(socket, host, tcpPort, (byte) 102);
        this.start_date = start_date;
        this.end_date = end_date;
        this.path = path;
    }

    public GetLogsTMS(Socket socket, String path, String start_date, String start_time,
        String end_date, String end_time, String CPL_UUID, String KDM_UUID) throws IOException {
        super(socket, (byte) 102);
        this.start_date = start_date;
        this.end_date = end_date;
        this.path = path;
    }

    protected byte[] createCmdData() {
        MessageData messageData = new MessageData();
        messageData.addElement(new StringElement(1, this.start_date.replaceAll("-", "")));
        messageData.addElement(new StringElement(2, this.end_date.replaceAll("-", "")));
        messageData.addElement(new IntegerElement(3, 1));
        return messageData.getElementsData();
    }

    protected String getResult(byte[] data) {
        String error =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>Can not get log path.</error>\n</response>";
        MessageData messageData = new MessageData(data);
        StringElement url = new StringElement(messageData.getElementByKey(5));
        String uri = "http://" + this.socket.getInetAddress().getHostAddress() + url.getValue();
        String saveFile =
            this.path + ((this.path.endsWith(File.separator)) ? "" : File.separator) + uri
                .substring(uri.lastIndexOf("/") + 1);
        String logFile = this.path + ((this.path.endsWith(File.separator)) ? "" : File.separator)
            + new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss").format(new Date()) + ".xml";
        if ((httpDownload(uri, saveFile)) && (unZip(saveFile, logFile))) {
            deleteFile(saveFile);
            String xml =
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >";
            xml = xml + "\n\t<auditlog_path>" + logFile + "</auditlog_path>";
            xml = xml + "\n</response>";
            logger.info("getResult xml = {}", xml);
            return xml;
        }
        return error;
    }

    protected String getErrorMsg(int errorCode) {
        String result = "Can not get log path.";
        switch (errorCode) {
            case 1:
                result = "Invalid from time";
                break;
            case 2:
                result = "Invalid to time";
                break;
            case 3:
                result = "Invalid type";
        }
        return result;
    }

    private boolean unZip(String zipFile, String saveToFile) {
        int length = 0;
        byte[] b = new byte[2048];
        try {
            File zFile = new File(zipFile);
            ZipFile zf = new ZipFile(zFile);
            ZipEntry ze = zf.getEntry("SMS.log");
            File f = new File(saveToFile);
            if (!(f.getParentFile().exists())) {
                f.getParentFile().mkdirs();
            }
            OutputStream outputStream = new FileOutputStream(f);
            InputStream inputStream = zf.getInputStream(ze);
            while ((length = inputStream.read(b)) > 0) {
                outputStream.write(b, 0, length);
            }
            inputStream.close();
            outputStream.close();
            zf.close();

            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    private boolean httpDownload(String httpUrl, String saveToFile) {
        int byteread = 0;

        URL url = null;
        try {
            url = new URL(httpUrl);
        } catch (MalformedURLException e1) {
            e1.printStackTrace();
            return false;
        }
        try {
            URLConnection conn = url.openConnection();
            InputStream inStream = conn.getInputStream();
            FileOutputStream fs = new FileOutputStream(saveToFile);

            byte[] buffer = new byte[1204];
            while ((byteread = inStream.read(buffer)) != -1) {
                fs.write(buffer, 0, byteread);
            }
            fs.close();
            return true;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            return false;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    private boolean deleteFile(String file) {
        try {
            Thread.sleep(1000L);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        return new File(file).delete();
    }

}
