package com.miaozhen.device.support.christienew.basecmd;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.charset.Charset;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class UtilTMSNew {
	private static final char[] HEX = "0123456789abcdef".toCharArray();
	private final Logger logger = LoggerFactory.getLogger(getClass());
	private LoginTMSNew login = null;

	public boolean login(Socket socket, String host, int tcpPort) {
		SocketAddress socketAddress = new InetSocketAddress(host, tcpPort);
		boolean result = false;
		try {
			if (socket == null || socket.isClosed()) {
				logger.info("socket is null");
				socket = new Socket();
			}
			if (!socket.isConnected()) {
				logger.info("socket is not Connected");
				socket.connect(socketAddress, 5000);
			    socket.setKeepAlive(true);
				socket.setSoTimeout(3000);
				this.login = new LoginTMSNew(socket, "service", "service");
				this.login.execute();
			}
			if (this.login == null) {
				this.login = new LoginTMSNew(socket, "service", "service");
				this.login.execute();
			}
			result = login.isSuccess();
		} catch (Exception e) {
			logger.error("IOException {} class {}", e.getMessage(), e
					.getClass().getName());
			e.printStackTrace();
		}
		return result;
	}

	public void closeSocket(Socket socket) {
		if (socket != null)
			try {
				if (!socket.isClosed())
					socket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		// finally {
		// if (socket != null) {
		// try {
		// if (!socket.isClosed())
		// socket.close();
		// } catch (IOException e) {
		// e.printStackTrace();
		// }
		// }
		// }
	}

	public static byte[] encodeDefiniteFormBER(int length) {
		byte[] ber;
		if (length < 128) {
			ber = new byte[1];
			ber[0] = (byte) (length & 0xFF);
		} else {
			int len = 1;
			int n = length;
			while (n > 0) {
				++len;
				n /= 256;
			}
			ber = new byte[len];
			ber[0] = (byte) (len - 1 | 0xFFFFFF80);
			for (int i = len - 1; i > 0; --i) {
				ber[i] = (byte) (length & 0xFF);
				length >>= 8;
			}
		}
		return ber;
	}

	public static int getDefiniteFormBERLength(byte berFirstByte) {
		int length = 0;
		if ((berFirstByte & 0xFFFFFF80) == 0)
			length = 1;
		else {
			length = (berFirstByte & 0x7F) + 1;
		}
		return length;
	}

	public static int decodeDefiniteFormBER(byte[] ber) {
		int length = 0;
		if ((ber[0] & 0xFFFFFF80) == 0) {
			length = ber[0] & 0xFF;
		} else {
			int len = ber.length;
			for (int i = 1; i < len; ++i) {
				length |= (ber[i] & 0xFF) << 8 * (len - i - 1);
			}
		}
		return length;
	}

	public static byte[] byteArrayCopy(byte[][] src) {
		int len = 0;
		for (int i = 0; i < src.length; ++i) {
			len += src[i].length;
		}
		byte[] result = new byte[len];
		int pos = 0;
		for (int i = 0; i < src.length; ++i) {
			System.arraycopy(src[i], 0, result, pos, src[i].length);
			pos += src[i].length;
		}
		return result;
	}

	public static String bytes2uuid(byte[] bytes) {
		if (bytes == null) {
			return null;
		}
		if (bytes.length == 0) {
			return "";
		}
		char[] chs = new char[bytes.length << 1];
		int i = 0;
		for (int k = 0; i < bytes.length; ++i) {
			chs[(k++)] = HEX[((bytes[i] & 0xF0) >> 4)];
			chs[(k++)] = HEX[(bytes[i] & 0xF)];
		}
		String shortuuid = new String(chs);
		String uuid = shortuuid.substring(0, 8) + "-"
				+ shortuuid.substring(8, 12) + "-"
				+ shortuuid.substring(12, 16) + "-"
				+ shortuuid.substring(16, 20) + "-"
				+ shortuuid.substring(20, 32);
		return uuid;
	}

	public static String byte2HexStr(byte[] b, String split) {
		String hs = "";
		String stmp = "";
		for (int n = 0; n < b.length; ++n) {
			stmp = Integer.toHexString(b[n] & 0xFF);
			if (stmp.length() == 1)
				hs = hs + "0" + stmp;
			else {
				hs = hs + stmp;
			}
			hs = hs + split;
		}
		return hs.toUpperCase();
	}

	public static int bigEndianToInt(byte[] value) {
		int result = 0;
		for (byte i = 0; i < value.length; i = (byte) (i + 1)) {
			result |= (value[i] & 0xFF) << (value.length - 1 - i) * 8;
		}
		return result;
	}

	public static byte[] intToBigEndian(int value) {
		byte[] result = new byte[4];
		result[0] = (byte) ((0xFF000000 & value) >> 24);
		result[1] = (byte) ((0xFF0000 & value) >> 16);
		result[2] = (byte) ((0xFF00 & value) >> 8);
		result[3] = (byte) (0xFF & value);
		return result;
	}

	public static int littleEndianToInt(byte[] value) {
		return bigEndianToInt(changeEndian(value));
	}

	public static byte[] intToLittleEndian(int value) {
		return changeEndian(intToBigEndian(value));
	}

	public static long bigEndianToLong(byte[] value) {
		long result = 0L;
		for (byte i = 0; i < value.length; i = (byte) (i + 1)) {
			result |= (value[i] & 0xFF) << (value.length - 1 - i) * 8;
		}
		return result;
	}

	public static byte[] longToBigEndian(long value) {
		long temp = value;
		byte[] b = new byte[8];
		for (int i = b.length - 1; i >= 0; --i) {
			b[i] = new Long(temp & 0xFF).byteValue();
			temp >>= 8;
		}
		return b;
	}

	public static long littleEndianToLong(byte[] value) {
		return bigEndianToLong(changeEndian(value));
	}

	public static byte[] longToLittleEndian(long value) {
		return changeEndian(longToBigEndian(value));
	}

	public static byte[] changeEndian(byte[] data) {
		int length = data.length;
		byte[] result = new byte[length];
		for (int i = 0; i < length; ++i) {
			result[i] = data[(length - i - 1)];
		}
		return result;
	}

	public static String readFileToString(String strFileName) {
		StringBuffer buf = null;
		BufferedReader breader = null;
		try {
			breader = new BufferedReader(new InputStreamReader(
					new FileInputStream(strFileName), Charset.forName("utf-8")));
			buf = new StringBuffer();
			while (breader.ready())
				buf.append((char) breader.read());
			breader.close();
		} catch (Exception localException) {
		}
		return buf.toString();
	}

	public static void main(String[] args) {
		byte[] ber = { -126, 39, -51 };
		System.out.println(decodeDefiniteFormBER(ber));
	}
}