package com.miaozhen.device.support.christienew.data;

import java.util.List;

import org.crifst.smi.christie.basecmd.Util;

public class LMessageData {
	private List<LElement> elements = new java.util.ArrayList();

	public LMessageData() {
	}

	public LMessageData(byte[] buffer) {
		this.elements = new java.util.ArrayList();
		separateElement(buffer);
	}

	public void addElement(LElement element) {
		this.elements.add(element);
	}

	public List<LElement> getElements() {
		return this.elements;
	}

	public LElement getElementByKey(int key) {
		LElement result = null;
		for (LElement element : this.elements) {
			if (element.getKey() == key) {
				result = element;
				break;
			}
		}
		return result;
	}

	private void separateElement(byte[] buffer) {
		if (buffer.length >= 6) {
			byte[] key = new byte[4];
			System.arraycopy(buffer, 0, key, 0, 4);
			int berLength = Util.getDefiniteFormBERLength(buffer[4]);
			
			byte[] ber = new byte[berLength];
			System.arraycopy(buffer, 4, ber, 0, berLength);
			//type
			byte[] bytype=new byte[1];
			System.arraycopy(buffer, 4+berLength, bytype, 0, 1);
			byte type=bytype[0];
			//type
			
			int dataLength = Util.decodeDefiniteFormBER(ber);
			//减去type所占的一个字节
			dataLength=dataLength-1;
			byte[] data = new byte[dataLength];
			System.arraycopy(buffer, 4 + berLength+1, data, 0, dataLength);
			this.elements.add(new LElement(key, ber,type, data));
			byte[] remainBuffer = new byte[buffer.length - (5 + berLength + dataLength)];
			System.arraycopy(buffer, 4 + berLength + dataLength+1, remainBuffer, 0, remainBuffer.length);
			separateElement(remainBuffer);
		}
	}

	public int[] caculateLen(byte[] buffer){
		int[] rs =new int[2];
		int bufferSize = buffer.length;
		int index=0;
		
		int temp_len = 0;
        int berlength = 0;
        
		if (bufferSize > index)
        {
            if ((buffer[0]&0xff) >= 0x80)
            {
                berlength =(buffer[index]&0xff) & 0x7f;
                //sanity check
                if (berlength > 3 || berlength == 0)
                {
                    berlength = 0;  // our error flag
                }
                else
                {
                    index++;
                    for (int i = 0; i < berlength; i++)
                    {
                    	int value = buffer[index++]&0xff;
                    	int value2=(temp_len << 8);
                        temp_len = value2 + value;
                    }
                    ++berlength;   // Increment to include the initial byte in the count.
                }
            }
            else
            {
            	 
                temp_len = buffer[index]&0xff;
                berlength = 1;  //the base case, one byte, 127 or under.
            }
        }
		rs[0]=berlength;
        rs[1]=temp_len;
        return rs;
	}
	
	public byte[] getElementsData() {
		byte[] result = new byte[0];
		for (LElement element : this.elements) {
			byte[][] a = { result, element.getElement() };
			result = Util.byteArrayCopy(a);
		}
		return result;
	}
}
