package com.miaozhen.device.support.dolby;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.miaozhen.device.support.smi.DeviceConstants;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.codehaus.xfire.client.Client;
import org.codehaus.xfire.transport.http.CommonsHttpMessageSender;
import org.crifst.project.tms.smi.dolby.api.Util;

import com.dolby.dcinema.ws.smi.v1.ContentManagementService_v1_0;
import com.dolby.dcinema.ws.smi.v1.ContentManagement_v1_0Client;
import com.dolby.dcinema.ws.smi.v1.Fault;
import com.dolby.dcinema.ws.smi.v1.schemas.common.LicenseStateEnum;
import com.dolby.dcinema.ws.smi.v1.schemas.contentmanagement.ClipInfo;
import com.dolby.dcinema.ws.smi.v1_0.*;

public class ContentManagementDolby {

	private String i;
	private ContentManagement_v1_0Client client;
	private ContentManagementService_v1_0 service;

	public ContentManagementDolby(HttpClientParams params, String ip) {
		this.i = Util.ip2AuditoriumNumber(ip);
		client = new ContentManagement_v1_0Client();
		service = client.getContentManagementService_v1_0("http://" + ip
				+ ":8080/dcinema/ws/smi/v1/ContentManagementService");
		Client.getInstance(service).setProperty(
				CommonsHttpMessageSender.HTTP_CLIENT_PARAMS, params);
	}

	public String getcpllist(String IP) {
		StringBuilder builder = null;
		GetClipsResponse response = null;
		GetClipsRequest request = new GetClipsRequest();
		request.setAuditoriumNumber(i);
		request.setContentStore(null);
		try {
			response = service.getClips(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");

			List<String> uuid_list = response.getClipId();
			for (Iterator<String> it = uuid_list.iterator(); it.hasNext();) {
				builder.append("\n\t<cpl_uuid>" + ((String) it.next())
						+ "</cpl_uuid>");
			}
			builder.append("\n</response>");
		} catch (Exception e) {
			e.printStackTrace();
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return builder.toString();
	}

	private ClipInfo getClipinfos(String IP, String UUID) {
		GetClipInfosResponse response = null;
		GetClipInfosRequest request = new GetClipInfosRequest();
		request.setAuditoriumNumber(i);
		request.setClipId(UUID);
		request.setContentStore(null);
		try {
			response = service.getClipInfos(request);
		} catch (Fault e) {
			e.printStackTrace();
		}

		if (response.getClipInfo().isEmpty()) {
			return null;
		}

		return ((ClipInfo) response.getClipInfo().get(0));
	}

	public String getcpl(String IP, String UUID) {
		GetCPLResponse response = null;
		StringBuilder builder = null;

		GetCPLRequest request = new GetCPLRequest();
		request.setClipId(UUID);
		try {
			response = service.getCPL(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			builder.append("\n\t<response_text>");
			String cpl = response.getClip();
			cpl = cpl.replace("&", "&amp;");
			cpl = cpl.replace("\"", "&quot;");
			cpl = cpl.replace("<", "&lt;");

			builder.append(cpl);

			builder.append("</response_text>");
			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String getcplinfo(String IP, String UUID) {
		StringBuilder builder = null;
		ClipInfo clipinfo = getClipinfos(IP, UUID);
		builder = new StringBuilder(
				"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
		builder.append("\n\t<cpl_uuid>" + clipinfo.getClipId() + "</cpl_uuid>");
		builder.append("\n\t<contenttitletext>" + clipinfo.getName()
				+ "</contenttitletext>");
		builder.append("\n\t<contentkind>" + clipinfo.getClipType()
				+ "</contentkind>");
		builder.append("\n\t<duration>" + clipinfo.getDuration()
				+ "</duration>");

		builder.append("\n\t<response_text>");

		return builder.toString();
	}

	public String validatecpl(String IP, String UUID) {

		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";

		String cpl_nonexist = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The content is not exists.</error>\n</response>";

		String kdm_invalid = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The KDM of the content is invalid.</error>\n</response>";

		String asset_missing = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The asset of the content is missing.</error>\n</response>";

		String unknow_error = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>Unknow error.</error>\n</response>";

		ClipInfo cpl = getClipinfos(IP, UUID);

		if (cpl == null)
			return cpl_nonexist;
		LicenseStateEnum licenseStateEnum = cpl.getLicensed().getValue();
		switch (licenseStateEnum) {
			case ABOUT_TO_EXPIRE:
				return isok;
			case ERROR:
				return kdm_invalid;
			case EXPIRED:
				return kdm_invalid;
			case EXTENSION:
				return isok;
			case FUTURE:
				return kdm_invalid;
			case MISSING:
				return kdm_invalid;
			case NOT_NEEDED:
				return isok;
			case NOT_RELEVANT:
				return kdm_invalid;
			case UNKNOWN:
				return unknow_error;
			case OK:
				return isok;
		}
		if ((cpl.isCorrupt()) || (cpl.isMissingData()))
			return asset_missing;
		if (cpl.isPlayable()) {
			return isok;
		}

		return unknow_error;
	}

	public String validateCplOverride(String IP, String UUID, Date playTime) {
		String successXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		String errorXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>%s</error>\n</response>";
		ClipInfo cpl = getClipinfos(IP, UUID);

		if (cpl == null) {
			return String.format(errorXml, UUID + DeviceConstants.SPL_MISS_CPL);
		}

		if ((cpl.isCorrupt()) || (cpl.isMissingData())) {
			return String.format(errorXml, UUID + DeviceConstants.SPL_MISS_CPL);
		}

		if(playTime == null){
			return successXml;
		}

		LicenseStateEnum licenseStateEnum = cpl.getLicensed().getValue();
		switch (licenseStateEnum) {
			case ABOUT_TO_EXPIRE:
				return successXml;
			case ERROR:
				return String.format(errorXml, DeviceConstants.CPL_MISS_KDM);
			case EXPIRED:
				return String.format(errorXml, DeviceConstants.EXPIRED_FUTUTE_KDM);
			case EXTENSION:
				return successXml;
			case FUTURE:
				return String.format(errorXml, DeviceConstants.EXPIRED_FUTUTE_KDM);
			case MISSING:
				return String.format(errorXml, DeviceConstants.CPL_MISS_KDM);
			case NOT_NEEDED:
				return successXml;
			case NOT_RELEVANT:
				return String.format(errorXml, DeviceConstants.VALIDATE_EXCEPTION);
			case UNKNOWN:
				return successXml;
			case OK:
				return successXml;
		}

		if (cpl.isPlayable()) {
			return successXml;
		}

		return String.format(errorXml, DeviceConstants.VALIDATE_EXCEPTION);
	}

	public String deletecontent(String IP, String UUID) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		DeleteContentRequest request = new DeleteContentRequest();
		request.setAuditoriumNumber(i);
		request.setContentId(UUID);
		request.setContentStore(null);
		request.setContentType(null);
		try {
			service.deleteContent(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return isok;
	}
}
