package com.miaozhen.device.support.dolby;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import com.miaozhen.device.support.smi.DeviceConstants;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.codehaus.xfire.client.Client;
import org.codehaus.xfire.transport.http.CommonsHttpMessageSender;
import org.crifst.project.tms.smi.dolby.api.Util;

import com.dolby.dcinema.ws.smi.v1.*;
import com.dolby.dcinema.ws.smi.v1.schemas.common.LicenseStateEnum;
import com.dolby.dcinema.ws.smi.v1.schemas.playbackcontrol.ScheduleMode;
import com.dolby.dcinema.ws.smi.v1.schemas.showmanagement.*;
import com.dolby.dcinema.ws.smi.v1_0.*;

public class ShowManagementDolby {

	private ShowManagement_v1_1Client client;
	private ShowManagementService_v1_1 service;

	private PlaybackControl_v1_0Client playbackControl_v1_0Client;
	private PlaybackControlService_v1_0 playbackControlService_v1_0;

	private String i;

	public ShowManagementDolby(HttpClientParams params, String ip) {
		this.i = Util.ip2AuditoriumNumber(ip);
		this.client = new ShowManagement_v1_1Client();
		this.service = client.getShowManagementService_v1_0("http://" + ip
				+ ":8080/dcinema/ws/smi/v1/ShowManagementService");
		Client.getInstance(service).setProperty(
				CommonsHttpMessageSender.HTTP_CLIENT_PARAMS, params);
		playbackControl_v1_0Client = new PlaybackControl_v1_0Client();
		playbackControlService_v1_0 = playbackControl_v1_0Client
				.getPlaybackControlService_v1_0("http://" + ip
						+ ":8080/dcinema/ws/smi/v1/PlaybackControlService");
		Client.getInstance(playbackControlService_v1_0).setProperty(
				CommonsHttpMessageSender.HTTP_CLIENT_PARAMS, params);
	}

	public String getspllist(String IP) {
		StringBuilder builder = null;
		GetShowsResponse response = null;
		GetShowsRequest request = new GetShowsRequest();
		try {
			response = service.getShows(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			List<String> uuid_list = response.getShowId();
			for (Iterator<String> it = uuid_list.iterator(); it.hasNext();) {
				builder.append("\n\t<show_uuid>" + ((String) it.next())
						+ "</show_uuid>");
			}
			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String getspl(String IP, String UUID) {
		StringBuilder builder = null;
		GetSPLResponse response = null;
		GetSPLRequest request = new GetSPLRequest();
		request.setShowId(UUID);
		try {
			response = service.getSPL(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			builder.append("\n\t<response_text>");
			String spl = response.getShow();
			spl = spl.replace("&", "&amp;");
			spl = spl.replace("\"", "&quot;");
			spl = spl.replace("<", "&lt;");
			spl = spl.replace(">", "&gt;");
			builder.append(spl);
			builder.append("</response_text>");
			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String validatespl(String IP, String UUID) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		String spl_nonexist = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The show is not exists.</error>\n</response>";
		String kdm_invalid = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The KDM of the show is invalid.</error>\n</response>";
		String asset_missing = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>The asset of the show is missing.</error>\n</response>";
		String unknow_error = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>Unknow error.</error>\n</response>";
		ShowInfo spl = getShowinfos(IP, UUID);
		if (spl == null)
			return spl_nonexist;
		LicenseStateEnum licenseStateEnum = spl.getLicensed().getValue();
		switch (licenseStateEnum) {
		case ABOUT_TO_EXPIRE:
			return isok;
		case ERROR:
			return kdm_invalid;
		case EXPIRED:
			return kdm_invalid;
		case EXTENSION:
			return isok;
		case FUTURE:
			return kdm_invalid;
		case MISSING:
			return kdm_invalid;
		case NOT_NEEDED:
			return isok;
		case NOT_RELEVANT:
			return kdm_invalid;
		case UNKNOWN:
			return unknow_error;
		case OK:
			return isok;
		}
		if ((spl.isCorrupt()) || (spl.isMissingData()))
			return asset_missing;
		if (spl.isPlayable()) {
			return isok;
		}

		return unknow_error;
	}

	public String validateSplOverride(String IP, String UUID, Date playTime) {
		String successXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		String errorXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>%s</error>\n</response>";
		ShowInfo spl = getShowinfos(IP, UUID);
		if (spl == null){
			return String.format(errorXml, DeviceConstants.VALIDATE_EXCEPTION);
		}
		LicenseStateEnum licenseStateEnum = spl.getLicensed().getValue();
		switch (licenseStateEnum) {
			case ABOUT_TO_EXPIRE:
				return successXml;
			case ERROR:
				return String.format(errorXml, DeviceConstants.CPL_MISS_KDM);
			case EXPIRED:
				return String.format(errorXml, DeviceConstants.EXPIRED_FUTUTE_KDM);
			case EXTENSION:
				return successXml;
			case FUTURE:
				return String.format(errorXml, DeviceConstants.EXPIRED_FUTUTE_KDM);
			case MISSING:
				return String.format(errorXml, DeviceConstants.CPL_MISS_KDM);
			case NOT_NEEDED:
				return successXml;
			case NOT_RELEVANT:
				return String.format(errorXml, DeviceConstants.VALIDATE_EXCEPTION);
			case UNKNOWN:
				return successXml;
			case OK:
				return successXml;
		}

		if (spl.isPlayable()) {
			return successXml;
		}

		return String.format(errorXml, DeviceConstants.VALIDATE_EXCEPTION);
	}


	public String deletespl(String IP, String UUID) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		DeleteShowRequest request = new DeleteShowRequest();
		request.setShowId(UUID);
		try {
			service.deleteShow(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return isok;
	}

	private ShowInfo getShowinfos(String IP, String UUID) {
		GetShowInfosResponse response = null;
		GetShowInfosRequest request = new GetShowInfosRequest();
		request.setAuditoriumNumber(i);
		request.setShowId(UUID);
		try {
			response = service.getShowInfos(request);
		} catch (Fault e) {
			e.printStackTrace();
		}

		if (response.getShowInfo().isEmpty()) {
			return null;
		}
		return ((ShowInfo) response.getShowInfo().get(0));
	}

	public String createspl(String IP, String spl) {
		CreateShowResponse response = null;
		CreateShowRequest request = new CreateShowRequest();
		request.setShow(spl);
		try {
			response = service.createShow(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<show_uuid>"
				+ response.getShowId() + "</show_uuid>" + "\n</response>";
	}

	public String getschedulelist(String IP) {
		StringBuilder builder = null;
		GetShowTimeInfosResponse response = null;
		GetShowTimeInfosRequest request = new GetShowTimeInfosRequest();
		request.setAuditoriumNumber(i);
		try {
			response = service.getShowTimeInfos(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			List<ShowTimeInfo> scheduleinfo_list = response.getShowTimeInfo();
			for (Iterator<ShowTimeInfo> it = scheduleinfo_list.iterator(); it
					.hasNext();) {
				ShowTimeInfo info = (ShowTimeInfo) it.next();
				builder.append("\n\t<schedule show_uuid=\"" + info.getShowId()
						+ "\" " + "iso_date_time=\"" + info.getStartTime()
						+ "\" >" + info.getShowTimeId() + "</schedule>");
			}
			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return builder.toString();
	}

	public String getschedule(String IP, String UUID) {
		StringBuilder builder = null;
		GetShowTimeInfosResponse response = null;
		GetShowTimeInfosRequest request = new GetShowTimeInfosRequest();
		request.setAuditoriumNumber(i);
		try {
			response = service.getShowTimeInfos(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			List<ShowTimeInfo> scheduleinfo_list = response.getShowTimeInfo();
			for (Iterator<ShowTimeInfo> it = scheduleinfo_list.iterator(); it
					.hasNext();) {
				ShowTimeInfo info = (ShowTimeInfo) it.next();
				if (info.getShowTimeId().matches(UUID)) {
					builder.append("\n\t<schedule show_uuid=\""
							+ info.getShowId() + "\" " + "iso_date_time=\""
							+ info.getStartTime() + "\" >"
							+ info.getShowTimeId() + "</schedule>");
				}
			}

			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String cancelschedule(String IP, String UUID) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		DeleteShowTimesRequest request = new DeleteShowTimesRequest();
		request.getShowTimeId().add(UUID);
		try {
			service.deleteShowTimes(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return isok;
	}

	public String getScheduleMode(String IP) {
		GetScheduleModeResponse response = null;
		GetScheduleModeRequest request = new GetScheduleModeRequest();
		try {
			response = playbackControlService_v1_0.getScheduleMode(request);
			switch (response.getScheduleMode().name()) {
			case "FULL_AUTOMATIC":
				return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<status enabled=\"true\"/>\n</response>";
			case "MANUAL":
				return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<status enabled=\"false\"/>\n</response>";
			case "SELECT_AUTOMATIC":
				return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<status enabled=\"true\"/>\n</response>";
			}

		} catch (Fault e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>Can not get scheduler status.</error>\n</response>";
	}

	public String enablescheduler(String IP) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		SetScheduleModeRequest request = new SetScheduleModeRequest();
		request.setScheduleMode(ScheduleMode.FULL_AUTOMATIC);
		try {
			playbackControlService_v1_0.setScheduleMode(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return isok;
	}

	public String disablescheduler(String IP) {
		String isok = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
		SetScheduleModeRequest request = new SetScheduleModeRequest();
		request.setScheduleMode(ScheduleMode.MANUAL);
		try {
			playbackControlService_v1_0.setScheduleMode(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return isok;
	}

	public String getcurrentschedule(String IP) {
		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>This function is not implemented now!</error>\n</response>";
	}

	public String createschedule(String IP, String date_time, String spl_UUID,
			String sch_UUID) {
		CreateShowTimesResponse response = null;
		CreateShowTimesRequest request = new CreateShowTimesRequest();
		ShowTimeRequest r = new ShowTimeRequest();
		r.setAuditoriumNumber(i);
		r.setShowId(spl_UUID);
		r.setShowTimeId(sch_UUID);
		try {
			XMLGregorianCalendar x = DatatypeFactory.newInstance()
					.newXMLGregorianCalendar(date_time);
			r.setStartTime(x);
			request.getShowTimeRequest().add(r);
			response = service.createShowTimes(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<schedule_uuid>"
				+ ((ShowTimeResponse) response.getShowTimeResponse().get(0))
						.getShowTimeId() + "</schedule_uuid>" + "\n</response>";
	}

	public String getcueinfos(String IP) {
		StringBuilder builder = null;
		GetCueInfosResponse response = null;
		String i = Util.ip2AuditoriumNumber(IP);
		GetCueInfosRequest request = new GetCueInfosRequest();
		request.setAuditoriumNumber(i);
		try {
			response = service.getCueInfos(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			List<CueInfo> cue_list = response.getCueInfo();
			for (Iterator<CueInfo> it = cue_list.iterator(); it.hasNext();) {
				CueInfo cueInfo = (CueInfo) it.next();
				builder.append("\n\t<automation_label uuid=\"urn:uuid:"
						+ cueInfo.getCueId() + "\">" + cueInfo.getName()
						+ "</automation_label>");
			}
			builder.append("\n</response>");
		} catch (Fault e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}
		return builder.toString();
	}
}