package com.miaozhen.device.support.dolby;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import com.miaozhen.device.support.DolbyServer;
import com.dolby.dcinema.ws.smi.v1.Fault;
import com.dolby.dcinema.ws.smi.v1.SystemManagementService_v1_1;
import com.dolby.dcinema.ws.smi.v1.SystemManagement_v1_1Client;
import com.dolby.dcinema.ws.smi.v1.schemas.systemmanagement.*;
import com.dolby.dcinema.ws.smi.v1_0.*;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.codehaus.xfire.client.Client;
import org.codehaus.xfire.transport.http.CommonsHttpMessageSender;
import org.crifst.project.tms.smi.dolby.api.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SystemManagementDolby {
    private final static String dolbyLogPath = "dolbyLog";
    private final Logger logger = LoggerFactory.getLogger(DolbyServer.class);
    private final Logger deviceLog = LoggerFactory.getLogger("device");

    private SystemManagement_v1_1Client client;
    private SystemManagementService_v1_1 service;
    private String i;

    public SystemManagementDolby(HttpClientParams params, String ip) {
        this.i = Util.ip2AuditoriumNumber(ip);
        client = new SystemManagement_v1_1Client();
        service = client.getSystemManagementService_v1_0(
            "http://" + ip + ":8080/dcinema/ws/smi/v1/SystemManagementService");
        Client.getInstance(service)
            .setProperty(CommonsHttpMessageSender.HTTP_CLIENT_PARAMS, params);
    }

    public String getauditlogs(String IP, String host, String user, String pass, String root,
        String start_date, String start_time, String end_date, String end_time, String CPL_UUID,
        String KDM_UUID) {
        String retrieve_uuid = null;
        GetAuditLogsResponse response = null;
        String i = Util.ip2AuditoriumNumber(IP);
        GetAuditLogsRequest request = new GetAuditLogsRequest();
        try {
            if (start_time.isEmpty()) {
                start_time = "00:00:01";
            }

            if (end_time.isEmpty()) {
                end_time = "00:00:01";
            }
            if (start_date.isEmpty()) {
                request.setStartDate(null);
            } else {
                XMLGregorianCalendar s = DatatypeFactory.newInstance()
                    .newXMLGregorianCalendar(start_date + "T" + start_time);
                request.setStartDate(s);
            }

            if (end_date.isEmpty()) {
                request.setEndDate(null);
            } else {
                XMLGregorianCalendar n = DatatypeFactory.newInstance()
                    .newXMLGregorianCalendar(end_date + "T" + end_time);
                request.setEndDate(n);
            }
            request.setAuditoriumNumber(i);
            request.setUsername(user);
            request.setPassword(pass);
            request.setUrl("ftp://" + host + "/" + root + "/");

            response = service.getAuditLogs(request);
            retrieve_uuid = (String) response.getLogRetrievalId().get(0);
        } catch (Exception e) {
            return
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
                    + e.getMessage() + "</error>" + "\n</response>";
        }
        SubThread st = new SubThread(IP, retrieve_uuid);
        st.start();
        try {
            st.join(720000L);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        st.interrupt();
        if (st.getResult()) {
            return "ok";
        }
        return "error";
    }

    public String geteventlogs(String IP, String host, String user, String pass, String root,
        String start_date, String start_time, String end_date, String end_time) {
        String isok =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
        String retrieve_uuid = null;
        GetSystemLogsResponse response = null;
        GetSystemLogsRequest request = new GetSystemLogsRequest();
        try {
            request.setAuditoriumNumber(i);
            request.setUsername(user);
            request.setPassword(pass);
            request.setUrl("ftp://" + host + "/" + root);
            response = service.getSystemLogs(request);
            retrieve_uuid = (String) response.getLogRetrievalId().get(0);
        } catch (Exception e) {
            return
                "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
                    + e.getMessage() + "</error>" + "\n</response>";
        }
        LogTransferStatusEnum s = null;
        do {
            s = geteventlogstatus(IP, retrieve_uuid);
        } while (s != LogTransferStatusEnum.FINISHED);

        return isok;
    }

    public LogTransferStatusEnum getcontentlogstatus(String IP, String UUID) {
        GetAuditLogsStatusResponse response = null;
        GetAuditLogsStatusRequest request = new GetAuditLogsStatusRequest();
        request.setAuditoriumNumber(i);
        request.setLogRetrievalId(UUID);
        try {
            response = service.getAuditLogsStatus(request);
        } catch (Fault e) {
            e.printStackTrace();
        }

        return ((LogStatusInfo) response.getAuditLogStatusInfo().iterator().next())
            .getTransferStatus().getValue();
    }

    public LogTransferStatusEnum geteventlogstatus(String IP, String UUID) {
        GetSystemLogsStatusResponse response = null;
        GetSystemLogsStatusRequest request = new GetSystemLogsStatusRequest();
        request.setAuditoriumNumber(i);
        request.setLogRetrievalId(UUID);
        try {
            response = service.getSystemLogsStatus(request);
        } catch (Fault e) {
            e.printStackTrace();
        }

        return ((LogStatusInfo) response.getSystemLogStatusInfo().iterator().next())
            .getTransferStatus().getValue();
    }


    class SubThread extends Thread {
        private String IP;
        private String retrieve_uuid;
        private boolean result = false;

        public SubThread(String paramString1, String paramString2) {
            this.IP = paramString1;
        }

        public boolean getResult() {
            return this.result;
        }

        public void run() {
            LogTransferStatusEnum s = null;
            do {
                s = SystemManagementDolby.this.getcontentlogstatus(this.IP, this.retrieve_uuid);
            } while (s != LogTransferStatusEnum.FINISHED);
            this.result = true;
        }
    }
}