package com.miaozhen.device.support.dolby;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.httpclient.params.HttpClientParams;
import org.codehaus.xfire.client.Client;
import org.codehaus.xfire.transport.http.CommonsHttpMessageSender;
import org.crifst.project.tms.smi.dolby.api.Util;

import com.dolby.dcinema.ws.smi.v1.TransferManagementService_v1_0;
import com.dolby.dcinema.ws.smi.v1.TransferManagement_v1_0Client;
import com.dolby.dcinema.ws.smi.v1.schemas.common.ContentStoreType;
import com.dolby.dcinema.ws.smi.v1.schemas.common.ContentStoreTypeEnum;
import com.dolby.dcinema.ws.smi.v1.schemas.common.ContentType;
import com.dolby.dcinema.ws.smi.v1.schemas.common.ContentTypeEnum;
import com.dolby.dcinema.ws.smi.v1.schemas.transfermanagement.*;
import com.dolby.dcinema.ws.smi.v1_0.*;

public class TransferManagementDolby {

	public static enum SmiTransportStateTypeEnum {
		QUEUED, RUNNING, FINISHED, ERROR;

		public String value() {
			return name();
		}

		public static SmiTransportStateTypeEnum fromValue(String v) {
			return valueOf(v);
		}
	}

	private String i;

	private TransferManagement_v1_0Client client;
	private TransferManagementService_v1_0 service;

	public TransferManagementDolby(HttpClientParams params, String ip) {
		this.i = Util.ip2AuditoriumNumber(ip);
		client = new TransferManagement_v1_0Client();
		service = client.getTransferManagementService_v1_0("http://" + ip
				+ ":8080/dcinema/ws/smi/v1/TransferManagementService");
		Client.getInstance(service).setProperty(
				CommonsHttpMessageSender.HTTP_CLIENT_PARAMS, params);
	}

	public String ingestcontent(String IP, String url, String user,
			String password, ContentTypeEnum contentType) {
		TransferContentResponse response = null;
		ContentType contenttype = new ContentType();
		contenttype.setValue(contentType);
		TransferContentRequest request = new TransferContentRequest();
		NetworkTransferType transfertype = new NetworkTransferType();
		transfertype.setValue(NetworkTransferTypeEnum.PULL);
		TransferProtocolType protocoltype = new TransferProtocolType();
		protocoltype.setValue(TransferProtocolTypeEnum.valueOf("FTP"));
		ContentStoreType storetype = new ContentStoreType();
		storetype.setValue(ContentStoreTypeEnum.MAINSTORE);

		request.setAuditoriumNumber(i);
		request.setContentType(contenttype);
		request.setTransferType(transfertype);
		request.setTransferProtocolType(protocoltype);
		request.setDestinationContentStore(storetype);
		request.setUrl(url);
		request.setUserName(user);
		request.setPassword(password);
		try {
			response = service.transferContent(request);
			// logger.info("response.getTransferId()=={}",
			// response.getTransferId());
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n\t<ingest_uuid>"
				+ response.getTransferId() + "</ingest_uuid>" + "\n</response>";
	}

	public String getingestlist(String IP) {
		StringBuilder builder = null;
		GetTransfersResponse response = null;
		GetTransfersRequest request = new GetTransfersRequest();
		request.setAuditoriumNumber(i);
		try {
			response = service.getTransfers(request);

			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			List<String> uuid_list = response.getTransferId();
			for (Iterator<String> it = uuid_list.iterator(); it.hasNext();) {
				builder.append("\n\t<ingest_uuid>" + ((String) it.next())
						+ "</ingest_uuid>");
			}
			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String cancelingest(String IP, String id) {
		CancelTransferRequest request = new CancelTransferRequest();
		request.setTransferId(id);
		try {
			service.cancelTransfer(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
	}

	public String getingeststatus(String IP, String id) {
		StringBuilder builder = null;
		GetTransferInfosResponse response = null;
		GetTransferInfosRequest request = new GetTransferInfosRequest();
		request.setAuditoriumNumber(i);
		request.setTransferId(id);
		try {
			response = service.getTransferInfos(request);
			builder = new StringBuilder(
					"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >");
			if (!(response.getTransferInfo().isEmpty())) {
				TransferInfo info = (TransferInfo) response.getTransferInfo()
						.get(0);
				builder.append("\n\t<ingest_status>");

				switch (info.getTransferStatus().getValue()) {
				case AWAITING_VERIFICATION:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.RUNNING.name()
							+ "</status>");
					break;
				case ERROR:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case CANCELLATION_PENDING:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case CANCELLED:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case EXTENSION:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case FINISHED:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.FINISHED.name()
							+ "</status>");
					break;
				case IN_PROGRESS:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.RUNNING.name()
							+ "</status>");
					break;
				case NOT_AVAILABLE:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case PAUSED:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case PENDING:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.QUEUED.name()
							+ "</status>");
					break;
				case UNKNOWN:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
					break;
				case VERIFYING:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.RUNNING.name()
							+ "</status>");
					break;
				default:
					builder.append("\n\t\t<status>"
							+ SmiTransportStateTypeEnum.ERROR.name()
							+ "</status>");
				}

				builder.append("\n\t\t<percent>" + info.getPercentageComplete()
						+ "</percent>");
				String ReasonInfo = "";
				if (info.getTransferStatusReason() != null) {
					ReasonInfo = info.getTransferStatusReason().getValue()
							.name();
				}
				builder.append("\n\t\t<description>"
						+ info.getTransferDescription() + ". ReasonInfo:"
						+ ReasonInfo + "</description>");
				builder.append("\n\t</ingest_status>");
			}

			builder.append("\n</response>");
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return builder.toString();
	}

	public String clearTransferHistoryRequest(String IP) {
		ClearTransferHistoryRequest request = new ClearTransferHistoryRequest();
		request.setAuditoriumNumber(i);
		try {
			service.clearTransferHistory(request);
		} catch (Exception e) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"ERROR\" version=\"2\" >\n\t<error>"
					+ e.getMessage() + "</error>" + "\n</response>";
		}

		return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<response status=\"OK\" version=\"2\" >\n</response>";
	}
}
