/**
 * SMI.java
 * @author Kamil
 * Created at 2013年11月30日
 */
package com.miaozhen.device.support.smi;

import java.io.*;

import javax.xml.bind.*;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.namespace.QName;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

/**
 * @author lzj
 * 
 * 2016年2月29日
 *
 * JAXBUtils.java
 */
public class JAXBUtils {
	
	public static <T> T unmarshalIgnoreNameSpace(InputStream is, Class<T> clazz) throws JAXBException {
		JAXBContext context = JAXBContext.newInstance(clazz);
		Unmarshaller unmarshaller = context.createUnmarshaller();
		SAXParserFactory sax = SAXParserFactory.newInstance();
		sax.setNamespaceAware(false);
		try {
			XMLReader xmlReader = sax.newSAXParser().getXMLReader();
			Source source = new SAXSource(xmlReader, new InputSource(is));
			JAXBElement<T> element = unmarshaller.unmarshal(source, clazz);
			return element.getValue();
		}catch(Exception e){
         e.printStackTrace();
		}
         return null;
	}

    public static <T> T unmarshalIgnoreNameSpace(String xml, Class<T> clazz) throws JAXBException {
        try {
            return unmarshalIgnoreNameSpace(new ByteArrayInputStream(xml.getBytes("UTF-8")), clazz);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }
    public static <T> T unmarshal(InputStream is, Class<T> clazz) throws JAXBException {
        JAXBContext context = JAXBContext.newInstance(clazz);
        Unmarshaller unmarshaller = context.createUnmarshaller();
        Source source = new StreamSource(is);
        JAXBElement<T> element = unmarshaller.unmarshal(source, clazz);
        return element.getValue();
    }


    public static <T> T unmarshal(String xml, Class<T> clazz) throws JAXBException{
		try {
			return unmarshal(new ByteArrayInputStream(xml.getBytes("UTF-8")), clazz);
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static <T> void marshal(OutputStream os, T object) throws JAXBException {
		@SuppressWarnings("unchecked")
		Class<T> clazz = (Class<T>) object.getClass();
		JAXBContext context = JAXBContext.newInstance(clazz);
		Marshaller marshaller = context.createMarshaller();
		
		Result result = new StreamResult(os);
		QName name = new QName(clazz.getAnnotation(XmlRootElement.class).name());
		JAXBElement<T> element = new JAXBElement<T>(name, clazz, object);
		marshaller.marshal(element, result);
	}
	
	public static <T> String marshal(T object) throws JAXBException {
		@SuppressWarnings("unchecked")
		Class<T> clazz = (Class<T>) object.getClass();
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		JAXBContext context = JAXBContext.newInstance(clazz);
		Marshaller marshaller = context.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);// 是否格式 化
		marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
		Result result = new StreamResult(os);
		QName name = new QName(clazz.getSimpleName());
		JAXBElement<T> element = new JAXBElement<T>(name, clazz, object);
		marshaller.marshal(element, result);
		try {
			String xml = os.toString("UTF-8");
			return xml;
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return null;
	}

}
