package com.miaozhen.device.util;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.cert.*;

import javax.security.auth.x500.X500Principal;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sun.misc.BASE64Encoder;

@SuppressWarnings("restriction")
public class DcCertReader {
	private X509Certificate cert;
	private PublicKey publickey;
	private Logger logger = LoggerFactory.getLogger(getClass());

	public DcCertReader() {
	}

	public DcCertReader(InputStream in) {
		try {
			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			logger.debug("DcCertReader() in.read(): {}", in.read());
			this.cert = ((X509Certificate) cf.generateCertificate(in));
			this.publickey = this.cert.getPublicKey();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			logger.error("DcCertReader() error", e);
		}
	}

	public DcCertReader(Certificate ct) {
		this.cert = ((X509Certificate) ct);
		this.publickey = ct.getPublicKey();
	}

	public X509Certificate getCert() {
		return this.cert;
	}

	public String getPublicKeyThumbprint() {
		String[] s = this.cert.getSubjectDN().toString().split("\"");
		return s[1];
	}

	public String getCertThumbprint() {
		MessageDigest sha = null;
		byte[] plainText = (byte[]) null;
		try {
			sha = MessageDigest.getInstance("SHA-1");
			plainText = this.cert.getTBSCertificate();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (CertificateEncodingException e) {
			e.printStackTrace();
		}

		sha.update(plainText);
		byte[] digest = sha.digest();
		return new BASE64Encoder().encode(digest);
	}

	public String toString() {
		return this.cert.toString();
	}

	public PublicKey getPublickey() {
		return this.publickey;
	}

	public String getIssuer() {
		X500Principal principal = this.cert.getIssuerX500Principal();

		return principal.getName("RFC1779").replace("+", "\\+")
				.replace("\"", "").replace("OID.2.5.4.46", "dnQualifier");
	}

	public String getSubject() {
		X500Principal principal = this.cert.getSubjectX500Principal();

		return principal.getName("RFC1779").replace("+", "\\+")
				.replace("\"", "").replace("OID.2.5.4.46", "dnQualifier");
	}

	public String getSerialNumber() {
		return this.cert.getSerialNumber().toString();
	}

	public static void main(String[] args) throws FileNotFoundException,
			CertificateException {
		FileInputStream in = new FileInputStream(
				"my_pem/testdevice.com-cert.pem");
		DcCertReader dcr = new DcCertReader(in);
		System.out.println(dcr.getIssuer());
		System.out.println(dcr.getPublicKeyThumbprint());
		System.out.println(dcr.getSubject());
		System.out.println(dcr.getCertThumbprint());
	}
}