package com.miaozhen.service.impl;


import com.miaozhen.device.ServerFactory;
import com.miaozhen.service.IServerLogServer;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.support.SonyServer;
import com.miaozhen.device.expection.DeviceException;
import com.miaozhen.tmssys.dto.Auditorium;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class ServerLogServerImpl implements IServerLogServer {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private final Map<Integer, Object> devices = new ConcurrentHashMap<>();

    @Override
    public void connect(Integer number, Auditorium auditorium) {
        logger.info("ServerLogServerImpl connect is start!!!");
        IDCServer iDCServer = (IDCServer) devices.get(number);
        releaseSonyServer(iDCServer);
        try {
            String serverType = auditorium.getServerType().trim();
            String serverIP = auditorium.getServerIp().trim();
            if (StringUtils.isBlank(serverIP) || StringUtils.isBlank(serverType)) {
                logger.info("auditorium data illegal, with ip:{}, serverType:{}", serverIP, serverType);
                return;
            }
            String userName = auditorium.getServerLoginUser();
            String password = auditorium.getServerLoginPassword();
            logger.info("connect serverType is:{},serverIP is:{},userName is:{},password is:{}",serverType,serverIP,userName,password);
            if (userName == null) {
                userName = StringUtils.EMPTY;
            }
            if (password == null) {
                password = StringUtils.EMPTY;
            }
            IDCServer server = ServerFactory.create(serverType, serverIP, userName, password);
            if (server == null) {
                logger.info("Can not create IDCServer(Type={}, IP={}).", serverType, serverIP);
                return;
            }else{
                logger.info("ServerLogServerImpl server serverType is:{}, getConnected is:{}",serverType,server.getConnected());
                if(server.getConnected()){//服务器连接成功则对象放入缓存
                    devices.put(number, server);
                    logger.info("ServerLogServerImpl connect is success!!! auditorium is:{}",auditorium.getAuditNumber());
                }
            }
        } catch (IllegalArgumentException e) {
            logger.error("The method connect(Integer number, Auditorium auditorium) in class {} occurred IllegalArgumentException."
                    , getClass().getSimpleName(), e);
        } catch (DeviceException e) {
            logger.error("When connecting the server of auditorium number {} occurred deviceException", number, e);
        }
    }

    @Override
    public Map<Integer, Object> getDevices() {
        return devices;
    }

    /**
     * Sony服务器的单独处理
     * @param iDCServer
     */
    private void releaseSonyServer(IDCServer iDCServer) {
        if (iDCServer != null && iDCServer instanceof SonyServer) {
            SonyServer serverSony = (SonyServer) iDCServer;
            try {
                serverSony.disconnect();
            } catch (DeviceException e) {
                logger.error("sony Server disconnect {} Exception {}", e.getClass().getName(), e.getMessage());
            }
        }
    }
}
