package com.miaozhen.task.job;

import com.google.common.base.Stopwatch;
import com.miaozhen.task.JobInterface;
import com.miaozhen.util.DateUtil;
import com.miaozhen.util.FileUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 清理30天前的项目日志
 **/
@Component
public class ClearLogJob implements JobInterface {

    Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public void execute() {
        logger.info("ClearLogJob clear log is start!");
        Stopwatch stopwatch = Stopwatch.createStarted();
        //清理日志 MiaozhenProbe_info.20200907.log
       String logsPath = FileUtil.getLogsDir();
        int daytime = Integer.parseInt(new SimpleDateFormat("yyyyMMdd").format(DateUtil.getNextDay(new Date(), -30)));
        if(new File(logsPath).isDirectory()){
            File[]logs=new File(logsPath).listFiles();
            for(int i=0;i<logs.length;i++){
                File log=logs[i];
                String filename = log.getName();
                String[] arr = filename.split("[.]");
                if(arr.length == 3 && ("log".equals(arr[2]))){
                    int start = filename.indexOf(".");
                    int end = filename.lastIndexOf(".");
                    if(start != end){
                        String date = filename.substring(start+1,end);
                        int dateTemp = Integer.parseInt(date);
                        //清理30天前的日志
                        if(dateTemp < daytime){
                            log.delete();
                        }
                    }
                }
            }
            logger.info("ClearLogJob clear log is over!");
        }

        //清理tempLog下面的临时日志  2020_02_27_09_39_00.xml
        logger.info("ClearLogJob clear tempLog is start!");
        String tempLogDir = FileUtil.getTempLogDir();
        int today = Integer.parseInt(new SimpleDateFormat("yyyyMMdd").format(new Date()));
        if(new File(tempLogDir).isDirectory()) {
            File[] tempLog = new File(tempLogDir).listFiles();
            for (int i = 0; i < tempLog.length; i++) {
                File log=tempLog[i];
                String filename = log.getName();
                String[] arr = filename.split("[_]");
                if(arr.length==6&&filename.contains(".xml")){
                   String logDate = arr[0]+arr[1]+arr[2];
                   int logDateTemp = Integer.parseInt(logDate);
//                    logger.info("ClearLogJob logDate is:{}",logDateTemp);
                   if(logDateTemp<today){
                       log.delete();
                   }
                }
            }
            logger.info("ClearLogJob clear tempLog is over!");
        }
        logger.info("ClearLogJob execute time is {}",stopwatch);
    }
}
