package com.miaozhen.task.job;

import com.google.common.base.Stopwatch;
import com.miaozhen.task.JobInterface;
import com.miaozhen.tmssys.TmsService;
import com.miaozhen.tmssys.dto.CinemaInfo;
import com.miaozhen.tmssys.dto.CinemaInfoRestResponse;
import com.miaozhen.util.FileUtil;
import com.miaozhen.util.HashMapUtil;
import com.miaozhen.util.JsonMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 获取放映服务器配置信息
 */
@Component
public class GetCinemaInfoJob implements JobInterface {

    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    TmsService tmsService;

    @Override
    public void execute() {
        logger.info("GetCinemaInfoJob is start!");
        Stopwatch stopwatch = Stopwatch.createStarted();
        CinemaInfoRestResponse resp = tmsService.cinemaInfo();
        if (resp != null) {
            CinemaInfo cinemaInfo = resp.getResult();
            if (cinemaInfo != null) {
                CinemaInfo cinemaInfoCache = (CinemaInfo)HashMapUtil.get(HashMapUtil.CACHE_KEY);
                if(cinemaInfoCache!=null){
                    String cinemaInfoJson = JsonMapper.getInstance().toJson(cinemaInfo);
                    String cinemaInfoCacheJson = JsonMapper.getInstance().toJson(cinemaInfoCache);
                     if(!cinemaInfoJson.equals(cinemaInfoCacheJson)){
                         HashMapUtil.put(HashMapUtil.CACHE_KEY,cinemaInfo);
                         FileUtil.setCinemaInfoToXml(cinemaInfo);
                     }
                }else{
                    HashMapUtil.put(HashMapUtil.CACHE_KEY,cinemaInfo);
                    FileUtil.setCinemaInfoToXml(cinemaInfo);
                }
            }
        }

        logger.info("GetCinemaInfoJob execute time is {}",stopwatch.stop());
    }
}
