package com.miaozhen.task.job;

import com.google.common.base.Stopwatch;
import com.miaozhen.device.IDCServer;
import com.miaozhen.device.smi.PlaybackStatus;
import com.miaozhen.device.support.ChristieServer;
import com.miaozhen.device.support.DolbyServer;
import com.miaozhen.service.CrashReportServer;
import com.miaozhen.service.IServerLogServer;
import com.miaozhen.task.JobInterface;
import com.miaozhen.tmssys.TmsService;
import com.miaozhen.tmssys.dto.*;
import com.miaozhen.util.*;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.net.NoRouteToHostException;
import java.net.SocketTimeoutException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 日志获取及上报
 */
@Component
public class ServerLogJob implements JobInterface {

    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    IServerLogServer serverLogServer;
    @Autowired
    TmsService tmsService;
    @Autowired
    CrashReportServer crashReportServer;

    private String tempLogPath;

    private String todayReportKey = "lastReportToday_";

    @PostConstruct
    public void init() {
        tempLogPath = FileUtil.getTempLogDir();
        //巴可需要提供一个存放日志的临时目录,bacro.jar中会用到
        System.setProperty("barco.log.path", tempLogPath);

        logger.info("ServerLogJob tempLogPath is:{}",tempLogPath);
        logger.info("ServerLogJob barco.log.path is:{}",System.getProperty("barco.log.path"));
    }


    @Override
    @Async
    public void execute() {
        logger.info("----------------------ServerLogJob is start---------------------");
        Stopwatch stopwatch = Stopwatch.createStarted();
        try {
            CinemaInfo cinemaInfo = (CinemaInfo)HashMapUtil.get(HashMapUtil.CACHE_KEY);
//                logger.info("cache cinemaInfo is:{}",cinemaInfo!=null?JsonMapper.getInstance().toJson(cinemaInfo):"is empty");
            if(cinemaInfo==null){
                cinemaInfo = FileUtil.getCinemaInfoFromXml();
            }
            if(cinemaInfo!=null){
                ConcurrentHashMap<String, ReportLog> reportMapTemp = FileUtil.getReportLogFromTemp(); // 日志上报临时缓存文件数据
                List<Auditorium> auditList =  cinemaInfo.getAuditList();
                if(auditList!=null&&auditList.size()>0){
                    for (Auditorium auditorium:auditList){
                        int auditNumber = auditorium.getAuditNumber();//影厅号
                        IDCServer device = (IDCServer) serverLogServer.getDevices().get(auditNumber);
/*                        if(device!=null){
                            logger.info("1、ServerLogJob device is:{}",device.getServerType());
                            logger.info("ServerLogJob device.getConnected() is:{}",device.getConnected());
                        }*/
                        if (device == null||(!device.getConnected())) {
                            logger.info("ServerLogJob is start connect!");
                            connect(auditNumber, cinemaInfo);
                        }
                        device = (IDCServer) serverLogServer.getDevices().get(auditNumber);
                        if(device!=null){
                            logger.info("2、ServerLogJob device is:{},getConnected  is:{}",device.getServerType(),device.getConnected());
                            PlaybackStatus playbackStatus = device.getPlaybackStatus();
                            if(playbackStatus!=null){
                                int state = playbackStatus.getState();
                                logger.info("ServerLogJob auditNumber is:{},state is:{}", auditNumber, state);
                                // 可以获取日志
                                if (state == PlaybackStatus.STATE_STOPPED||state == PlaybackStatus.STATE_WAITINGFORLOADING) {
                                    logger.info("ServerLogJob can fetch log:state is:{}",state);
                                    if (device != null) {
                                        String serverType = device.getServerType();//服务器类型
                                        logger.info("serverType is:{}",serverType);
                                        /**
                                         * 检测昨天开始最近21天日志是否已经上报
                                         */
                                        int day = -21;
                                        for(int i=0;i>=day;i--){
                                            Date startDate = DateUtil.getNextDay(new Date(), i);
                                            Date endDate = DateUtil.getNextHour(startDate, 24);
                                            String startDateStr = DateUtil.format(startDate, DateUtil.FORMAT_DATE);
                                            String endDateStr = DateUtil.format(endDate, DateUtil.FORMAT_DATE);
                                            boolean isReportFlag = false;
                                            String reportLogKey = startDateStr + "&" + auditNumber;//某厅某天日历key值

                                            if(i == 0){//当天时时数据
                                                if (reportMapTemp != null && reportMapTemp.size() > 0) {
                                                    ReportLog rl = reportMapTemp.get(todayReportKey + auditNumber);
                                                    if (rl == null || rl.getReportTime()==null) {//可以上报
                                                        isReportFlag = true;
                                                    }else{
                                                        Date lastReport = rl.getReportTime();
                                                        int  tomDiffMinutes = DateUtil.differentMinutes(lastReport, new Date());
                                                        if (tomDiffMinutes >= 60) {//1小时
                                                            isReportFlag = true;
                                                        }
                                                    }
                                                }else{//缓存没有记录可以上报
                                                    isReportFlag = true;
                                                }
                                                if(isSmallHours()){
                                                    startDate = DateUtil.getNextDay(new Date(), i-1);
                                                    endDate = DateUtil.getNextHour(startDate, 24);
                                                    startDateStr = DateUtil.format(startDate, DateUtil.FORMAT_DATE);
                                                    endDateStr = DateUtil.format(endDate, DateUtil.FORMAT_DATE);
                                                }
                                            }else if(isFetchLogTime()){//获取历史数据
                                                if (reportMapTemp != null && reportMapTemp.size() > 0) {
                                                    if (reportMapTemp.get(reportLogKey) == null) {//可以上报
                                                        isReportFlag = true;
                                                    }
                                                }else{//缓存没有记录可以上报
                                                    isReportFlag = true;
                                                }
                                            }

                                            logger.info("ServerLogJob isReportFlag is {},auditNumber is:{},startDate is:{}",isReportFlag,auditNumber,startDateStr);
                                            if (isReportFlag) {   //可以上报
                                                String logContent = "";
                                                if (device instanceof DolbyServer){
                                                    tempLogPath = cinemaInfo.getFtp().getRootPath();
                                                }
                                                //通过接口获取服务器日志
                                                logContent = device.getAuditLog(cinemaInfo.getFtp().getFtpIP(), cinemaInfo.getFtp().getFtpUserName(),
                                                        cinemaInfo.getFtp().getFtpPassword(), tempLogPath, startDateStr, endDateStr);
                                                if (StringUtils.isBlank(logContent) && device instanceof ChristieServer) {//科室
                                                    boolean flag = device.ejectSPL("");
                                                    logContent = device.getAuditLog(cinemaInfo.getFtp().getFtpIP(), cinemaInfo.getFtp().getFtpUserName(),
                                                            cinemaInfo.getFtp().getFtpPassword(), tempLogPath, startDateStr, endDateStr);
                                                }
                                                if (StringUtils.isNotBlank(logContent)) {
                                                    /***************##################日志上报接口处理####################**************/
                                                    //获取到日志则进行上报，连接
                                                    Stopwatch Stopwatch2 = Stopwatch.createStarted();
                                                    //文件夹路径：ooh/yyyy-MM-dd
//                                                    String sftpMsmDir = configBean.getSftpMsmDir() + startDateStr;
//                                                    if (i == 0) {
//                                                        //实时日志路径
//                                                        sftpMsmDir = configBean.getSegment() + startDateStr;
//                                                    }
                                                    //文件名字：IP_服务器类型_HH-mm-ss-SSS-随机字符串.json.md5
                                                    String fileName = FileUtils.getIp() + "_" + device.getIP() + "_" + serverType + "_" +
                                                            DateUtil.format(new Date(), "HH-mm-ss-SSS") + "-" + MD5Utils.getUUID() + ".json";
                                                    logger.info("ServerLogJob fileName is:{}", fileName);
//                                                    String result = FileUtils.sendData(logContent, serverType, cinemaInfo.getCode(), auditNumber, device.getIP(),
//                                                            sftpMsmDir, fileName, startDateStr, FileUtil.getCinemaInfoMd5());
                                                    boolean success = false;
//                                                    if (StringUtils.isNotBlank(result) && result.contains("success")) {
//                                                        success = true;
//                                                    }
                                                   /* UploadBean bean = new UploadBean();
                                                    bean.setServerType(serverType);
                                                    bean.setServerIp(device.getIP());
                                                    bean.setAuditNumber(auditNumber);
                                                    bean.setCinemaCode(cinemaInfo.getCode());
                                                    bean.setDate(startDateStr);
                                                    bean.setContent(logContent);
                                                    bean.setConfigMd5(FileUtil.getCinemaInfoMd5());
                                                    //上传
                                                    boolean success = FileUtils.uploadString(JSONObject.fromObject(bean).toString(), fileName, sftpMsmDir);
                                                    */
                                                    logger.info("ServerLogJob fileName is:{},upload status is:{},uploadFTP time is {}",fileName,success,Stopwatch2.stop());
                                                    //关闭
                                                    FileUtils.disConnectSftp();
                                                    /***************##################日志上报接口处理####################**************/

                                                    //根据result结果判定上报成功则在本地缓存做记录
                                                    if (success) {
                                                        logger.info("ServerLogJob fileName is:{},success is:{}",fileName,success);
                                                        ReportLog reportLog = new ReportLog();
                                                        reportLog.setAuditNumber(auditNumber);
                                                        reportLog.setData(startDateStr);
                                                        if(i == 0){
                                                            reportLog.setReportTime(new Date());
                                                            reportMapTemp.put(todayReportKey + auditNumber,reportLog);
                                                        }else{
                                                            reportMapTemp.put(reportLogKey, reportLog);
                                                        }
                                                        FileUtil.writeReportLogToTemp(reportMapTemp);//写入缓存文件
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (NoRouteToHostException e){
            e.printStackTrace();
        }catch (SocketTimeoutException e){
            e.printStackTrace();
        }catch (Exception e) {
            logger.info("异常出现",e);
//            crashReportServer.uploadErrorLogFileToFTP();
            e.printStackTrace();
        }
        logger.info("ServerLogJob execute time is {}",stopwatch.stop());
    }

    /**
     * 连接服务器
     * @param auditNumber
     * @param cinemaInfo
     */
    public void connect(int auditNumber, CinemaInfo cinemaInfo) {
        logger.info("ServerLogJob connect is start!!!!");
        List<Auditorium> auditoriumList = cinemaInfo.getAuditList();
        Map<Integer, Auditorium> auditMap = new HashedMap();
        if (auditoriumList != null && auditoriumList.size() > 0) {
            for (Auditorium auditorium : auditoriumList) {
                auditMap.put(auditorium.getAuditNumber(), auditorium);
            }
        }
        Auditorium audit = auditMap.get(auditNumber);
        logger.info("audit is:{}",JsonMapper.getInstance().toJson(audit));
        if (audit != null) {
            serverLogServer.connect(auditNumber, audit);
        }
    }

    /**
     * 大于6点钟
     * @return
     */
    public boolean isFetchLogTime(){
        boolean flag = false;
        String dateStr = DateUtil.format(new Date(),DateUtil.FORMAT_DATETIME);  //yyyy-MM-dd HH:mm
        String hourAndMin = dateStr.split(" ")[1];
        int hour = Integer.parseInt(hourAndMin.split(":")[0]);
        if(hour>6){
            flag = true;
        }
        return flag;
    }

    /**
     * 小于6点钟
     * @return
     */
    public boolean isSmallHours(){
        boolean flag = false;
        String dateStr = DateUtil.format(new Date(),DateUtil.FORMAT_DATETIME);  //yyyy-MM-dd HH:mm
        String hourAndMin = dateStr.split(" ")[1];
        int hour = Integer.parseInt(hourAndMin.split(":")[0]);
        if(hour<6){
            flag = true;
        }
        return flag;
    }

}
