package com.miaozhen.util;

import java.io.*;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import com.miaozhen.Application;
import com.miaozhen.tmssys.dto.CinemaInfo;
import com.miaozhen.tmssys.dto.PlaybackStatus;
import com.miaozhen.tmssys.dto.ReportLog;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.system.ApplicationHome;

public class FileUtil {

    private static Logger logger = LoggerFactory.getLogger(FileUtil.class);


    public static boolean deleteFolder(String sPath) {
        boolean flag = false;
        File file = new File(sPath);
        // 判断目录或文件是否存在
        if (!file.exists()) {  // 不存在返回 false
            return flag;
        } else {
            // 判断是否为文件
            if (file.isFile()) {  // 为文件时调用删除文件方法
                return deleteFile(sPath);
            } else {  // 为目录时调用删除目录方法
                return deleteDirectory(sPath);
            }
        }
    }

    /**
     * 删除单个文件
     *
     * @param sPath 被删除文件的文件名
     * @return 单个文件删除成功返回true，否则返回false
     */
    public static boolean deleteFile(String sPath) {
        boolean flag = false;
        File file = new File(sPath);
        // 路径为文件且不为空则进行删除
        if (file.isFile() && file.exists()) {
            file.delete();
            flag = true;
        }
        return flag;
    }


    /**
     * 删除目录（文件夹）以及目录下的文件
     *
     * @param sPath 被删除目录的文件路径
     * @return 目录删除成功返回true，否则返回false
     */
    public static boolean deleteDirectory(String sPath) {
        //如果sPath不以文件分隔符结尾，自动添加文件分隔符
        if (!sPath.endsWith(File.separator)) {
            sPath = sPath + File.separator;
        }
        File dirFile = new File(sPath);
        //如果dir对应的文件不存在，或者不是一个目录，则退出
        if (!dirFile.exists() || !dirFile.isDirectory()) {
            return false;
        }
        boolean flag = true;
        //删除文件夹下的所有文件(包括子目录)
        File[] files = dirFile.listFiles();
        for (int i = 0; i < files.length; i++) {
            //删除子文件
            if (files[i].isFile()) {
                flag = deleteFile(files[i].getAbsolutePath());
                if (!flag) break;
            } //删除子目录
            else {
                flag = deleteDirectory(files[i].getAbsolutePath());
                if (!flag) break;
            }
        }
        if (!flag) return false;
        //删除当前目录
        if (dirFile.delete()) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean isHave(String sPath) {
        File file = new File(sPath.substring(0, sPath.lastIndexOf(".")));
        // 判断目录或文件是否存在
        if (file.isDirectory() && file.exists()) {  // 不存在返回 false
            return true;
        }
        return false;
    }

    public static void setDirPermissions(String strPath) {
        try {
            Runtime.getRuntime().exec("chmod -R 777  " + strPath);
        } catch (IOException e) {
        }
    }

    /**
     * 获取项目产生日志的目录
     * @return
     */
    public static String getLogsDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "logs";
        return tempFilePath;
    }
    /**
     * 获取创建的文件夹serverlogs的临时目录
     * @return
     */
    public static String getServerlogsDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "serverlogs";
        File file = new File(tempFilePath);
        if (!file.exists()) {
            file.mkdir();
        }
        return tempFilePath;
    }
    /**
     * 获取创建的文件夹的临时目录
     * @return
     */
    public static String getTempLogDir() {
        ApplicationHome home = new ApplicationHome(Application.class);
        File jarFile = home.getSource();
        String path = jarFile.getParentFile().getPath();
        String tempFilePath = path + File.separator + "tempLog";
        File file = new File(tempFilePath);
        if (!file.exists()) {
            file.mkdir();
        }
//        setDirPermissions(tempFilePath);
        return tempFilePath;
    }

    /**
     * 将每天日志上报记录写到本地缓存文件
     *
     * @param map
     */
    public synchronized static void writeReportLogToTemp(ConcurrentHashMap<String, ReportLog> map) {
        String path = getServerlogsDir() + File.separator + "ReportLog";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(map);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 将影院相关信息CinemaInfo写到本地缓存文件
     *
     * @param cinemaInfo
     */
    public synchronized static void writeCinemaInfoToTemp(CinemaInfo cinemaInfo) {
        String path = getServerlogsDir() + File.separator + "CinemaInfo";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(cinemaInfo);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 从临时文件获取每天日志上报记录
     *
     * @return
     */
    public static ConcurrentHashMap<String, ReportLog> getReportLogFromTemp() {
        ConcurrentHashMap<String, ReportLog> result = new ConcurrentHashMap();
        String path = getServerlogsDir() + File.separator + "ReportLog";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (ConcurrentHashMap<String, ReportLog>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }

    /**
     * 从临时文件获取影院相关信息CinemaInfo
     *
     * @return
     */
    public static CinemaInfo getCinemaInfoFromTemp() {
        CinemaInfo result = null;
        String path = getServerlogsDir() + File.separator + "CinemaInfo";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (CinemaInfo) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    /**
     * 从临时文件获取影院相关信息CinemaInfo  【测试使用】
     *
     * @return
     */
    public static CinemaInfo getCinemaInfoTestFromTemp() {
        CinemaInfo result = null;
        String path = "D:\\serverlogs\\CinemaInfo";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (CinemaInfo) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    /**
     * 将影厅服务器状态PlaybackStatus写到本地缓存文件  【测试使用】
     *
     * @param list
     */
    public synchronized static void writePlaybackStatusToTemp(List<PlaybackStatus> list) {
        String path = getServerlogsDir() + File.separator + "PlaybackStatus";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(list);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    /**
     * 从临时文件获取PlaybackStatus  【测试使用】
     *
     * @return
     */
    public static List<PlaybackStatus> getPlaybackStatusFromTemp() {
        List<PlaybackStatus> result = null;
//        String path ="D:\\serverlogs\\PlaybackStatus";
        String path = getServerlogsDir() + File.separator + "PlaybackStatus";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (List<PlaybackStatus>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }

    /**
     * 从缓存xml中读取信息并转换成对象
     * @return
     * @throws Exception
     */
    public static CinemaInfo getCinemaInfoFromXml() {
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            File file = new File(path);
            String xml = "";
            if(file.exists()){
                ByteArrayOutputStream bos=new ByteArrayOutputStream();
                FileInputStream fis=new FileInputStream(file);
                int a=0;
                while((a=fis.read())>0){
                    bos.write(a);
                }
                xml=new String(bos.toByteArray());
            }
            if(StringUtils.isNotBlank(xml)){
                return JaxbUtil.converyToJavaBean(xml,CinemaInfo.class);
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return null;
    }

    public static String getCinemaInfoMd5() {
        String md5 = "";
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            File file = new File(path);
            if (file.exists()) {
                md5 = DigestUtils.md5Hex(new FileInputStream(file));
                System.out.println("config md5 = " + md5);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return md5;
    }

    /**
     * 把对象信息转换xml并写入缓存文件中
     * @param cinemaInfo
     * @throws Exception
     */
    public static void setCinemaInfoToXml(CinemaInfo cinemaInfo){
        try {
            String path = getServerlogsDir() + File.separator + "CinemaInfo.xml";
            String xml = JaxbUtil.convertToXml(cinemaInfo,"UTF-8");//转换xml
            FileOutputStream fos = new FileOutputStream(path);
            fos.write(xml.getBytes());
        }catch (Exception e){
            e.printStackTrace();
        }

    }

    /**
     * 是否上报了2019-12-01之后的所有日志记录
     * @param map
     */
    public synchronized static void writeIsReportHistoryLogToTemp(ConcurrentHashMap<String, Boolean> map) {
        String path = getServerlogsDir() + File.separator + "IsReportHistoryLog";
        FileOutputStream fos = null;
        ObjectOutputStream os = null;
        try {
            File f = new File(path);
            if (!f.exists()) {
                f.createNewFile();
            }
            fos = new FileOutputStream(path);
            os = new ObjectOutputStream(fos);
            os.writeObject(map);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fos != null) {
                    fos.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 获取是否上报完毕2019-12-01之后的所有日志记录标识
     * @return
     */
    public static ConcurrentHashMap<String, Boolean> getIsReportHistoryLogFromTemp() {
        ConcurrentHashMap<String, Boolean> result = new ConcurrentHashMap();
        String path = getServerlogsDir() + File.separator + "IsReportHistoryLog";
        if (!new File(path).exists()) {
            return result;
        }
        ObjectInputStream is = null;
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(path);
            is = new ObjectInputStream(fis);
            result = (ConcurrentHashMap<String, Boolean>) is.readObject();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e1) {
                }
            }
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e1) {
                }
            }
        }
        return result;
    }
    public static void main(String[] args) {
    }
}

