package com.miaozhen.util;


import com.jcraft.jsch.*;
import com.miaozhen.bean.PostBean;
import com.miaozhen.bean.UploadBean;
import de.schlichtherle.util.zip.ZipEntry;
import de.schlichtherle.util.zip.ZipOutputStream;
import net.sf.json.JSONObject;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.http.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

public class FileUtils {
    private static String JAR_MD5_VALUE;
    private static String ip;
    private static ChannelSftp sftp = null;
    private static Session sshSession = null;
    private static Logger logger = LoggerFactory.getLogger(FileUtils.class);
    public final static String[] hexDigits = {"0", "1", "2", "3", "4", "5",
            "6", "7", "8", "9", "a", "b", "c", "d", "e", "f"};


    /**
     *
     * @param content 日志内容
     * @param serverType 服务器类型
     * @param cinemaCode 影院专资ID
     * @param auditNumber 影厅号
     * @param deviceIp 服务器ip地址
     * @param fileDir dts 文件夹路径
     * @param date 日期
     */
    public static String sendData(String content, String serverType, String cinemaCode, int auditNumber, String deviceIp,
                                  String fileDir, String fileName,String date, String configMd5) {

        String body = "", json, bodyMd5, key = "a0d714a3-2529-4571-b546-7c81b3da64e1", result = "", path;
        if (fileDir.endsWith("/")) {
            path = fileDir;
        } else {
            path = fileDir + "/";
        }
        logger.info("ServerLogJob path is{}:fileName is:{}", path, fileName);
        UploadBean bean = new UploadBean();
        bean.setServerType(serverType);
        bean.setServerIp(deviceIp);
        bean.setAuditNumber(auditNumber);
        bean.setCinemaCode(cinemaCode);
        bean.setDate(date);
        bean.setContent(content);
        bean.setSdkMd5(JAR_MD5_VALUE);
        bean.setConfigMd5(configMd5);
        try {
            body = RSA.encryptByPublicKey(JSONObject.fromObject(bean).toString().getBytes(), RSA.PUBLIC_KEY);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (StringUtils.isEmpty(body)) {
            logger.error("body is empty ! return");
            return result;
        }
        //md5是content加密之后的MD5
        bodyMd5 = DigestUtils.md5Hex(body);
        PostBean post = new PostBean();
        post.setBody(body);
        post.setMd5(bodyMd5);
        post.setFilename(fileName);
        post.setPath(path);
        //token=md5(fileName+path+key)
        post.setToken(DigestUtils.md5Hex(fileName + path + key));
        json = JSONObject.fromObject(post).toString();
        result = HttpUtils.postDataAsBody("https://bbttrans-ooh.cn.miaozhen.com/bbt/upload", json);
        logger.info("post logContent result :{}", result);
        return result;
    }


    public static void setJarMd5Value(String md5){
        if(JAR_MD5_VALUE != null){
            return;
        }
        JAR_MD5_VALUE = md5;
    }

    public static String getJarMd5Value(){
        return JAR_MD5_VALUE;
    }

    public static String getIp() {
        if (TextUtils.isEmpty(ip)) {
            String data = HttpUtils.getData("http://pv.sohu.com/cityjson");
            if (TextUtils.isEmpty(data))
                return "";
            String[] js = data.split("[=;]");
            if (js.length <= 1)
                return "";
            JSONObject jsonObject = JSONObject.fromObject(js[1]);
            ip = jsonObject.getString("cip");
        }
        return ip;
    }
    /**
     * sftp服务器连接
     *
     * @param
     */
    public static ChannelSftp connectSftp(String host, Integer port, String userName, String password) {
        JSch jsch = new JSch();
        try {
            jsch.getSession(userName, host, port);
            sshSession = jsch.getSession(userName, host, port);
            sshSession.setPassword(password);
            Properties properties = new Properties();
            properties.put("StrictHostKeyChecking", "no");
            sshSession.setConfig(properties);
            sshSession.connect();
            Channel channel = sshSession.openChannel("sftp");
            channel.connect();
            sftp = (ChannelSftp) channel;

        } catch (JSchException e) {
            e.printStackTrace();
        }
        return sftp;
    }

    /**
     * 设置远程目录权限
     */
    public static void setPermission(Integer permission, String remoteStr) {
        try {
            sftp.chmod(permission, remoteStr);
        } catch (SftpException e) {
            e.printStackTrace();
        }
    }

    /**
     * 上传文件到sftp
     *
     * @param
     * @return
     */
    public static void uploadFile(String FileDir, String remoteDir) {
        File dir = new File(FileDir);
        File[] files = dir.listFiles();
        String path = remoteDir;
        String[] folders = path.split("/");
        try {
            sftp.cd("/");
            for (String folder : folders) {
                if (folder.length() > 0) {
                    try {
                        sftp.cd(folder);
                    } catch (SftpException e) {
                        sftp.mkdir(folder);
                        sftp.cd(folder);
                    }
                }
            }
            logger.info("当前所在目录:" + path);
            if (dir.isDirectory()) {
                for (int i = 0; i < files.length; i++) {
                    if (files[i].isFile()) {
                        String fileName = files[i].getName();
                        logger.info("读取文件:" + fileName);
                        FileInputStream fis = new FileInputStream(files[i]);
                        sftp.put(fis, fileName);
                        fis.close();
                    }
                }
                logger.info(path + "/目录下文件上传完毕");
            } else if (dir.isFile()) {
                logger.info("打标识操作所在目录:" + path);
                File flag = new File(FileDir);
                logger.info("读取文件:" + flag.getName());
                FileInputStream fis = new FileInputStream(flag);
                sftp.put(fis, flag.getName());
                fis.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 上传字符串到sftp
     * 上传成功就返回true，反之false
     * @param
     * @return
     */
    public static boolean uploadString(String data, String fileName, String remoteDir) {
        String path = remoteDir, md5 = DigestUtils.md5Hex(data), reName;
        String[] folders = path.split("/");
        logger.info(path + "->准备上传，文件名：{}，文件content长度：{}", fileName,data.length());
        boolean success;
        try {
            sftp.cd("/");
            for (String folder : folders) {
                if (folder.length() > 0) {
                    try {
                        sftp.cd(folder);
                    } catch (SftpException e) {
                        sftp.mkdir(folder);
                        sftp.cd(folder);
                    }
                }
            }
            logger.info("当前所在目录:" + path);
            ByteArrayInputStream inputStream = new ByteArrayInputStream(data.getBytes());
            sftp.put(inputStream, fileName);
            inputStream.close();
            logger.info(path + "->字符串上传完毕,路径：{},文件名：{}", remoteDir, fileName);
            //上传完成文件，修改文件名
            reName = fileName + ".success." + md5;
            sftp.rename(fileName, reName);
            logger.info(path + "->修改文件名成功：{}", reName);
            success = true;
        } catch (Exception e) {
            e.printStackTrace();
            success = false;
        }
        return success;
    }

    /**
     * 关闭sftp连接
     *
     * @param
     */
    public static void disConnectSftp() {
        if (sftp != null) {
            if (sftp.isConnected()) {
                sftp.disconnect();
                sftp = null;
            }
        }
        if (sshSession != null) {
            if (sshSession.isConnected()) {
                sshSession.disconnect();
                sshSession = null;
            }
        }
    }


    /**
     * 创建zip文件
     *
     * @param filePath    --the file's path
     * @param zipFilePath
     * @throws UnsupportedEncodingException
     * @throws NullPointerException
     * @author zhangxihe
     * @date 2010-12-28
     */
    public static void createZipFile(String filePath, String zipFilePath,
                                     String encoding) throws NullPointerException,
            UnsupportedEncodingException {
        FileOutputStream fos = null;
        ZipOutputStream zos = null;
        try {
            fos = new FileOutputStream(zipFilePath);
            zos = "".equals(encoding) ? new ZipOutputStream(fos)
                    : new ZipOutputStream(fos, "utf-8");
            // zos = new ZipOutputStream(fos,encoding);
            writeZipFile(new File(filePath), zos, "");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } finally {
            try {
                if (zos != null) {
                    zos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void writeZipFile(File f, ZipOutputStream zos,
                                    String hiberarchy) {
        if (f.exists()) {
            if (f.isDirectory()) {
                hiberarchy += f.getName() + "/";
                File[] fif = f.listFiles();
                for (int i = 0; i < fif.length; i++) {
                    writeZipFile(fif[i], zos, hiberarchy);
                }
            } else {
                FileInputStream fis = null;
                try {
                    fis = new FileInputStream(f);
                    ZipEntry ze = new ZipEntry(hiberarchy + f.getName());
                    zos.putNextEntry(ze);
                    byte[] b = new byte[1024];
                    while (fis.read(b) != -1) {
                        zos.write(b);
                        b = new byte[1024];
                    }
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (fis != null) {
                            fis.close();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * @param folderPath 设定文件
     * @return void 返回类型
     * @throws
     * @Title: delFolder
     * @Description: 删除文件夹
     * @author zhangxihe
     * @date 2010-12-29
     */
    public static void delFolder(String folderPath) {
        try {
            String filePath = folderPath;
            File delPath = new File(filePath);
            String[] childs = delPath.list();
            if (null == childs || childs.length <= 0) {
                delPath.delete();
            } else {
                for (String childName : childs) {
                    String childPath = delPath.getPath() + File.separator
                            + childName;
                    // childPath +=childName;
                    File file = new File(childPath);
                    if (file.exists() && file.isFile()) {
                        file.delete();
                    } else if (file.exists() && file.isDirectory()) {
                        delFolder(childPath);
                    }
                }
                delPath.delete();
            }
        } catch (Exception e) {
            System.out.println("删除文件夹操作出错");
            e.printStackTrace();
        }
    }

    /**
     * 在工程下创建新的文件夹用于存放postbuy报表供用户下载
     *
     * @param @param folderPath 设定文件
     * @return void 返回类型
     * @throws
     * @Title: newFolder
     * @Description: 在工程下创建新的文件夹用于存放postbuy报表供用户下载
     */
    public static void newFolder(String folderPath) throws Exception {
        File myFilePath = new File(folderPath);
        if (!myFilePath.exists() && !myFilePath.isDirectory()) {
            if (!myFilePath.mkdir()) {
                throw new Exception("创建存放postbuy报表的文件夹失败！");
            }
        }
    }

    public static File doZipExcel(String excelPath, String zipPath,
                                  boolean isDeleteExcel) {
        if ((excelPath == null) || ("".equals(excelPath.trim()))) {
            throw new RuntimeException("excelPath为空!");
        }
        if ((zipPath == null) || ("".equals(zipPath.trim()))) {
            throw new RuntimeException("zipPath为空!");
        }

        Map<String, String> excelFileNamePath = new HashMap<String, String>();
        File excelFile = new File(excelPath);

        if (excelFile.isDirectory()) {
            File[] files = excelFile.listFiles();
            for (int i = 0; i < files.length; i++)
                if (files[i].isFile()) {
                    String fileName = files[i].getName();
                    if (fileName.toLowerCase().endsWith(".xlsx")) {
                        String excelFilePath = files[i].getAbsolutePath();
                        String excelName = files[i].getName();
                        excelFileNamePath.put(excelName, excelFilePath);
                    }
                }
        } else {
            String excelFilePath = excelFile.getAbsolutePath();
            String excelName = excelFile.getName();
            excelFileNamePath.put(excelName, excelFilePath);
        }
        File zipFile = new File(zipPath);

        if (zipFile.isDirectory()) {
            if (zipPath.endsWith(File.separator))
                zipPath = zipPath + ".zip";
            else {
                zipPath = zipPath + File.separator + ".zip";
            }
        }
        try {
            FileOutputStream fos = new FileOutputStream(zipPath);
            ZipOutputStream zipos = new ZipOutputStream(
                    new BufferedOutputStream(fos));
            Iterator<Map.Entry<String, String>> excelFileNamePathItor = excelFileNamePath
                    .entrySet().iterator();
            while (excelFileNamePathItor.hasNext()) {
                Map.Entry<String, String> excelFileNamePathEntry = excelFileNamePathItor
                        .next();
                String excelFileName = excelFileNamePathEntry.getKey();
                String excelFilePath = excelFileNamePathEntry
                        .getValue();
                zipos.putNextEntry(new ZipEntry(excelFileName));
                BufferedInputStream bis = new BufferedInputStream(
                        new FileInputStream(excelFilePath));
                int c;
                while ((c = bis.read()) != -1) {
                    zipos.write(c);
                    zipos.flush();
                }
                bis.close();
            }
            zipos.close();
        } catch (Exception e) {
            throw new RuntimeException("excel文件压缩出错!");
        }

        if (isDeleteExcel) {
            Iterator<Map.Entry<String, String>> excelFileNamePathItor = excelFileNamePath
                    .entrySet().iterator();
            while (excelFileNamePathItor.hasNext()) {
                Map.Entry<String, String> excelFileNamePathEntry = excelFileNamePathItor
                        .next();
                String excelFilePath = excelFileNamePathEntry
                        .getValue();
                File excelF = new File(excelFilePath);
                if (excelF.exists())
                    excelF.delete();
            }
        }

        return zipFile;
    }

    /**
     * 获取文件的MD5值
     *
     * @param file
     * @return
     */
    public static String getFileMD5(File file) {
        String md5 = null;
        FileInputStream fis = null;
        FileChannel fileChannel = null;
        try {
            fis = new FileInputStream(file);
            fileChannel = fis.getChannel();
            MappedByteBuffer byteBuffer = fileChannel.map(FileChannel.MapMode.READ_ONLY, 0, file.length());

            try {
                MessageDigest md = MessageDigest.getInstance("MD5");
                md.update(byteBuffer);
                md5 = byteArrayToHexString(md.digest());
            } catch (NoSuchAlgorithmException e) {

                e.printStackTrace();
            }
        } catch (FileNotFoundException e) {

            e.printStackTrace();
        } catch (IOException e) {

            e.printStackTrace();
        } finally {
            try {
                fileChannel.close();
                fis.close();
            } catch (IOException e) {

                e.printStackTrace();
            }
        }

        return md5;
    }

    /**
     * 字节数组转十六进制字符串
     *
     * @param digest
     * @return
     */
    private static String byteArrayToHexString(byte[] digest) {

        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < digest.length; i++) {
            buffer.append(byteToHexString(digest[i]));
        }
        return buffer.toString();
    }

    /**
     * 字节转十六进制字符串
     *
     * @param b
     * @return
     */
    private static String byteToHexString(byte b) {
        //  int d1 = n/16;
        int d1 = (b & 0xf0) >> 4;

        //   int d2 = n%16;
        int d2 = b & 0xf;
        return hexDigits[d1] + hexDigits[d2];
    }


    public static boolean rsync(String src, String desc) {
        try {
            logger.info("rsync -r --delete " + src + " " + desc);
            Process p = Runtime.getRuntime().exec(
                    "rsync -r --delete  " + src + " " + desc);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }
    }

    public static boolean rsync_no_delete(String src, String desc) {
        try {
            logger.info("rsync -r  " + src + " " + desc);
            Process p = Runtime.getRuntime().exec(
                    "rsync -r   " + src + " " + desc);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }
    }

    public static boolean touch(String src) {
        try {
            logger.info("touch" + src);
            Process p = Runtime.getRuntime().exec(
                    "touch  " + src);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }
    }

    public static boolean mv(String src, String desc) {
        try {
            logger.info("mv " + src + "  " + desc);
            Process p = Runtime.getRuntime().exec(
                    "mv  " + src + "  " + desc);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }
    }

    public static int rsfilecount() {
        try {
            logger.info("exec-----");
            Process p = Runtime.getRuntime().exec(
                    "ssh root@d65.mzhen.cn sh /usr/local/gytest/run.sh");
            p.waitFor();
            p.destroy();
            return p.exitValue();
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return 0;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return 0;
        }
    }

    /**
     * @param cmd
     * @throws
     * @Title: 执行远程shell，并返回输出
     * @author yujiwei
     * @date 2015-03-31
     */
    public static String execCMD(String cmd) throws Exception {
        String result = "1";
        Runtime r = Runtime.getRuntime();
        try {
            Process p = r.exec(cmd);
            result = loadStream(p.getInputStream());
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        return result;
    }

    /**
     * @param in
     * @throws
     * @Title: 获取流读取到内存中
     * @author yujiwei
     * @date 2015-03-31
     */
    public static String loadStream(InputStream in) throws IOException {
        int ptr = 0;
        in = new BufferedInputStream(in);
        StringBuffer buffer = new StringBuffer();
        while ((ptr = in.read()) != -1) {
            buffer.append((char) ptr);
        }
        return buffer.toString();
    }
//	public static void main(String[] args) throws Exception {
//		String result = execCMD("ssh root@d65.mzhen.cn sh /usr/local/gytest/run.sh");
//		System.out.println(result.trim());
//		System.out.println("over");
//	}


    /**
     * scp 拷贝文件
     *
     * @param src
     * @param desc
     * @return
     */
    public static boolean scp(String src, String desc) {
        try {
            logger.info("scp " + src + " " + desc);
            Process p = Runtime.getRuntime().exec(
                    "scp " + src + " " + desc);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }
    }

    /**
     * @param server
     * @param path
     * @return
     */
    public static boolean mkdir(String server, String path) {
        try {
            String cmd = "ssh " + server + " " +
                    "[ ! -d " + path + " ] && mkdir -p " + path + "";
            logger.info(cmd);
            Process p = Runtime.getRuntime().exec(cmd);
            p.waitFor();
            p.destroy();
            return true;
        } catch (IOException e) {
            logger.info(e.getMessage(), e);
            return false;
        } catch (InterruptedException e) {
            logger.info(e.getMessage(), e);
            return false;
        }

    }


    public static boolean createDirectory(String directory) {
        File directoryFile = new File(directory);

        if (directoryFile.exists() && directoryFile.isDirectory())
            return true;
        else {
            return directoryFile.mkdirs();
        }
    }

    public static void appendFileContent(String fileName, String content) {
        try {
            // 打开一个随机访问文件流，按读写方式
            RandomAccessFile randomFile = new RandomAccessFile(fileName, "rw");
            // 文件长度，字节数
            long fileLength = randomFile.length();
            //将写文件指针移到文件尾。
            randomFile.seek(fileLength);
            randomFile.writeBytes(content);
            randomFile.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
