package com.bobocorn.MiaozhenProbe;


import com.miaozhen.util.JsonMapper;
import org.apache.commons.lang.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

public class JaxbUtil {

    /**
     * java对象转换为xml文件
     *
     * @param load    java对象.Class
     * @return xml文件的String
     * @throws JAXBException
     */
    public static String beanToXml(Object obj, Class<?> load) throws JAXBException {
        JAXBContext context = JAXBContext.newInstance(load);
        Marshaller marshaller = context.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, false);
        StringWriter writer = new StringWriter();
        marshaller.marshal(obj, writer);
        return writer.toString();
    }

    /**
     * xml文件配置转换为对象
     *
     * @param xmlPath xml文件路径
     * @param load    java对象.Class
     * @return java对象
     * @throws JAXBException
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    public static <T> T xmlToBean(String xmlPath, Class<T> load) throws JAXBException, IOException {
        JAXBContext context = JAXBContext.newInstance(load);
        Unmarshaller unmarshaller = context.createUnmarshaller();
        return (T) unmarshaller.unmarshal(new StringReader(xmlPath));
    }

    /**
     * JavaBean转换成xml 默认编码UTF-8
     *
     * @param obj
     * @return
     */
    public static String convertToXml(Object obj) {
        return convertToXml(obj, "UTF-8");
    }

    /**
     * JavaBean转换成xml
     *
     * @param obj
     * @param encoding
     * @return
     */
    public static String convertToXml(Object obj, String encoding) {
        String result = null;
        try {
            JAXBContext context = JAXBContext.newInstance(obj.getClass());
            Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.setProperty(Marshaller.JAXB_ENCODING, encoding);
            //去掉生成xml的默认报文头
            marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);
            StringWriter writer = new StringWriter();
            writer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "\n    ");
            marshaller.marshal(obj, writer);
            result = writer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * JavaBean转换成xml去除xml声明部分
     *
     * @param obj
     * @param encoding
     * @return
     */
    public static String convertToXmlIgnoreXmlHead(Object obj, String encoding) {
        String result = null;
        try {
            JAXBContext context = JAXBContext.newInstance(obj.getClass());
            Marshaller marshaller = context.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.setProperty(Marshaller.JAXB_ENCODING, encoding);
            marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);
            StringWriter writer = new StringWriter();
            marshaller.marshal(obj, writer);
            result = writer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * xml转换成JavaBean
     *
     * @param xml
     * @param c
     * @return
     */
    @SuppressWarnings("unchecked")
    public static <T> T converyToJavaBean(String xml, Class<T> c) {
        T t = null;
        try {
            JAXBContext context = JAXBContext.newInstance(c);
            Unmarshaller unmarshaller = context.createUnmarshaller();
            t = (T) unmarshaller.unmarshal(new StringReader(xml));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return t;
    }

    private static OutputFormat createPrettyPrint() {
        OutputFormat format = new OutputFormat();
        //format.setIndentSize(2);
        format.setNewLineAfterDeclaration(false);
        format.setNewlines(true);
        format.setTrimText(false);
        format.setPadText(false);
        return format;
    }

    /**
     *
     * @Title: formatXml
     * @Description: 格式化xml方法
     * @param str
     * @return
     * @throws Exception
     */
    public static String formatXml(String str) throws Exception {
        Document document = null;
        document = DocumentHelper.parseText(str);
        // 格式化输出格式
        OutputFormat format = createPrettyPrint();
        format.setEncoding("UTF-8");
        StringWriter writer = new StringWriter();
        // 格式化输出流
        XMLWriter xmlWriter = new XMLWriter(writer, format);
        // 将document写入到输出流
        xmlWriter.write(document);
        xmlWriter.close();
        return writer.toString();
    }

    public static void main(String[] args) {
        Auditorium audit1 = new Auditorium();
        audit1.setAuditNumber(1);
        audit1.setServerIp("192.168.2.8");
        audit1.setServerLoginPassword("wode999");
        audit1.setServerLoginUser("username001");
        audit1.setServerType("GDC");
        Auditorium audit2 = new Auditorium();
        audit2.setAuditNumber(2);
        audit2.setServerIp("192.168.2.112");
        audit2.setServerLoginPassword("pass555");
        audit2.setServerLoginUser("username002");
        audit2.setServerType("Sony");
        audit2.setAuditNumber(3);
        audit2.setServerIp("192.168.2.99");
        audit2.setServerLoginPassword("pass33");
        audit2.setServerLoginUser("username003");
        audit2.setServerType("Sony");
        List<Auditorium> list = new ArrayList<Auditorium>();
        list.add(audit1);
        list.add(audit2);
        Ftp ftp = new Ftp();
        ftp.setFtpIP("192.168.8.9");
        ftp.setFtpUserName("tmsUser");
        ftp.setFtpPassword("tms");
        ftp.setRootPath("media/kdm");
        CinemaInfo cinemaInfo = new CinemaInfo();
        cinemaInfo.setAuditList(list);
        cinemaInfo.setCode("123456789");
        cinemaInfo.setFtp(ftp);


        String json = JsonMapper.getInstance().toJson(list);
        String xml = convertToXml(cinemaInfo,"UTF-8");//转换xml
        System.out.println(xml);
        CinemaInfo cinema = converyToJavaBean(xml,CinemaInfo.class);
        System.out.println(JsonMapper.getInstance().toJson(cinema));
        try {
            String path = "D:\\feng\\CinemaInfo.xml";
            File file = new File(path);
            String xmlTemp = "";
            if(file.exists()){
                ByteArrayOutputStream bos=new ByteArrayOutputStream();
                FileInputStream fis=new FileInputStream(file);
                int a=0;
                while((a=fis.read())>0){
                    bos.write(a);
                }
                xmlTemp=new String(bos.toByteArray());
            }
            if(StringUtils.isNotBlank(xmlTemp)){
                CinemaInfo cinemaInfo1 = converyToJavaBean(xmlTemp, CinemaInfo.class);
                System.out.println("====>"+JsonMapper.getInstance().toJson(cinemaInfo1));
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }
}